# Copyright (C) 2003  CAMP
# Copyright (C) 2010  Argonne National Laboratory
# Please see the accompanying LICENSE file for further information.

"""
Python wrapper functions for the ``C`` package:
Linear Algebra PACKage (LAPACK)
"""

import numpy as np

from gpaw import debug
import _gpaw
from gpaw.utilities.tools import tri2full
from gpaw.utilities.blas import gemm


def diagonalize(a, w):
    """Diagonalize a symmetric/hermitian matrix.

    Uses dsyevd/zheevd to diagonalize symmetric/hermitian matrix
    `a`. The eigenvectors are returned in the rows of `a`, and the
    eigenvalues in `w` in ascending order. Only the lower triangle of
    `a` is considered."""

    assert a.flags.contiguous
    assert w.flags.contiguous
    assert a.dtype in [float, complex]
    assert w.dtype == float
    n = len(a)
    assert a.shape == (n, n)
    assert w.shape == (n,)

    info = _gpaw.diagonalize(a, w)
    if info != 0:
        raise RuntimeError('diagonalize error: %d' % info)


def diagonalize_mr3(a, w, z):
    """Diagonalize a symmetric/hermitian matrix.

    Uses dsyevr/zheevr to diagonalize symmetric/hermitian matrix
    `a`. The eigenvectors are returned in the rows of `z`, and the
    eigenvalues in `w` in ascending order. Only the lower triangle of
    `a` is considered."""

    assert a.flags.contiguous
    assert w.flags.contiguous
    assert z.flags.contiguous
    assert a.dtype in [float, complex]
    assert w.dtype == float
    assert z.dtype == a.dtype
    n = len(a)
    assert a.shape == (n, n)
    assert w.shape == (n,)
    assert z.shape == (n, n)
    info = _gpaw.diagonalize_mr3(a, w, z)
    if info != 0:
        raise RuntimeError('diagonalize_mr3 error: %d' % info)


def general_diagonalize(a, w, b, iu=None):
    """Diagonalize a generalized symmetric/hermitian matrix
    
    A * x = (lambda) * B * x,

    where `lambda` is the eigenvalue and `A` and `B` are the 
    matrices corresponding to `a` and `b`, respectively.

    If `iu` is `None`:
    Uses dsygvd/zhegvd to diagonalize symmetric/hermitian matrix
    `a`. The eigenvectors are returned in the rows of `a`, and the
    eigenvalues in `w` in ascending order. Only the lower triangle of
    `a` is considered.

    If `iu` is not `None`:
    Uses dsygvx/zhegvx to find the eigenvalues of 1 through `iu`.
    Stores the eigenvectors in `z` and the eigenvalues in `w` in
    ascending order.
    """

    assert a.flags.contiguous
    assert w.flags.contiguous
    assert a.dtype in [float, complex]
    assert w.dtype == float
    n = len(a)
    assert a.shape == (n, n)
    assert w.shape == (n,)
    assert b.flags.contiguous
    assert b.dtype == a.dtype
    assert b.shape == a.shape

    if iu is not None:
        z = np.zeros((n, n), dtype=a.dtype)
        assert z.flags.contiguous

    w[:1] = 42

    if iu is None:
        info = _gpaw.general_diagonalize(a, w, b)
    else:
        info = _gpaw.general_diagonalize(a, w, b, z, iu)
        a[:] = z

    if info != 0 or n > 0 and w[0] == 42:
        raise RuntimeError('general_diagonalize error: %d' % info)


def inverse_cholesky(a):
    """Calculate the inverse of the Cholesky decomposition of
    a symmetric/hermitian positive definite matrix `a`.

    Uses dpotrf/zpotrf to calculate the decomposition and then
    dtrtri/ztrtri for the inversion"""

    assert a.flags.contiguous
    assert a.dtype in [float, complex]
    n = len(a)
    assert a.shape == (n, n)

    info = _gpaw.inverse_cholesky(a)
    if info != 0:
        raise RuntimeError('inverse_cholesky error: %d' % info)


def inverse_general(a):
    assert a.dtype in [float, complex]
    n = len(a)
    assert a.shape == (n, n)
    info = _gpaw.inverse_general(a)
    if info != 0:
        raise RuntimeError('inverse_general error: %d' % info)


def inverse_symmetric(a):
    assert a.dtype in [float, complex]
    n = len(a)
    assert a.shape == (n, n)
    info = _gpaw.inverse_symmetric(a)
    tri2full(a, 'L', 'symm')
    if info != 0:
        raise RuntimeError('inverse_symmetric: %d' % info)


def right_eigenvectors(a, w, v):
    """Get right eigenvectors and eigenvalues from a square matrix
    using LAPACK dgeev.

    The right eigenvector corresponding to eigenvalue w[i] is v[i]."""

    assert a.flags.contiguous
    assert w.flags.contiguous
    assert v.flags.contiguous
    assert a.dtype == float
    assert w.dtype == float
    assert v.dtype == float
    n = len(a)
    assert a.shape == (n, n)
    assert w.shape == (n,)
    assert w.shape == (n, n)
    return _gpaw.right_eigenvectors(a, w, v)


def pm(M):
    """print a matrix or a vector in mathematica style"""
    string = ''
    s = M.shape
    if len(s) > 1:
        n, m = s
        string += '{'
        for i in range(n):
            string += '{'
            for j in range(m):
                string += str(M[i, j])
                if j == m - 1:
                    string += '}'
                else:
                    string += ','
            if i == n - 1:
                string += '}'
            else:
                string += ','
    else:
        n = s[0]
        string += '{'
        for i in range(n):
            string += str(M[i])
            if i == n - 1:
                string += '}'
            else:
                string += ','
    return string


def sqrt_matrix(a, preserve=False):
    """Get the sqrt of a symmetric matrix a (diagonalize is used).
    The matrix is kept if preserve=True, a=sqrt(a) otherwise."""
    n = len(a)
    if debug:
        assert a.flags.contiguous
        assert a.dtype == float
        assert a.shape == (n, n)
    if preserve:
        b = a.copy()
    else:
        b = a

    # diagonalize to get the form b = Z * D * Z^T
    # where D is diagonal
    D = np.empty((n,))
    diagonalize(b, D)
    ZT = b.copy()
    Z = np.transpose(b)

    # c = Z * sqrt(D)
    c = Z * np.sqrt(D)

    # sqrt(b) = c * Z^T
    gemm(1., ZT, np.ascontiguousarray(c), 0., b)
    return b



"""gpaw-python Segmentation faults
when gpaw-python and numpy are linked to different blas"""
from __future__ import division
from math import sqrt
from ase import Atoms
from gpaw import GPAW
from gpaw import ConvergenceError

kpts = (2, 1, 1)
a = 1.42
c = 3.355
 
# AB stack
atoms = Atoms('C4',
              [(1 / 3, 1 / 3, 0),
               (2 / 3, 2 / 3, 0),
               (0, 0, 0.5),
               (1 / 3, 1 / 3, 0.5)],
              pbc=(1, 1, 1))

atoms.set_cell([(sqrt(3) * a / 2, 3 / 2 * a, 0),
                (-sqrt(3) * a / 2, 3 / 2 * a, 0),
                (0, 0, 2 * c)],
               scale_atoms=True)

calc = GPAW(gpts=(8, 8, 20), nbands=9, kpts=kpts, maxiter=1)

atoms.set_calculator(calc)

try:
    pot = atoms.get_potential_energy()
except ConvergenceError:
    pass



from ase import Atoms, Atom
from ase.vibrations.placzek import Placzek
from ase.parallel import world, DummyMPI

from gpaw import GPAW
from gpaw.lrtddft.kssingle import KSSingles
from gpaw.test import equal

"""
Test Placzek Raman implementation
"""

txt = '-'
txt = None
load = True
load = False
xc = 'LDA'

# ---------------------------------------------------
# run

R = 0.7  # approx. experimental bond length
a = 4.0
c = 5.0
H2 = Atoms([Atom('H', (a / 2, a / 2, (c - R) / 2)),
            Atom('H', (a / 2, a / 2, (c + R) / 2))],
           cell=(a, a, c))

calc = GPAW(xc=xc, nbands=3, spinpol=False, eigensolver='rmm-diis', txt=txt)
H2.set_calculator(calc)
H2.get_potential_energy()

gsname = exname = 'rraman'
exkwargs={'eps':0.0, 'jend':1}
pz = Placzek(H2, KSSingles, gsname=gsname, exname=exname,
             exkwargs=exkwargs)
pz.run()

# ---------------------------------------------------
# check

# check size
kss = KSSingles('rraman-d0.010.eq.ex.gz')
assert(len(kss) == 1)

om = 5
pz = Placzek(H2, KSSingles, gsname=gsname, exname=exname,
                   verbose=True,)
ai = pz.absolute_intensity(omega=om)[-1]
equal(ai, 301.659994669, 1e-3) # earlier obtained value
i = pz.intensity(omega=om)[-1]
equal(i, 7.87430831452e-05, 1e-11) # earlier obtained value
pz.summary(omega=5, method='frederiksen')

# parallel ------------------------

if world.size > 1 and world.rank == 0:
    # single core
    comm = DummyMPI()
    pzsi = Placzek(H2, KSSingles, gsname=gsname, exname=exname,
                   comm=comm, verbose=True,)
    isi = pzsi.intensity(omega=om)[-1]
    equal(isi, i, 1e-11)




import numpy as np

from ase.units import Bohr, Hartree

from gpaw import GPAW
from gpaw.external import ConstantElectricField
from gpaw.lcaotddft.hamiltonian import KickHamiltonian
from gpaw.lcaotddft.hamiltonian import TimeDependentHamiltonian
from gpaw.lcaotddft.logger import TDDFTLogger
from gpaw.lcaotddft.propagators import create_propagator
from gpaw.tddft.units import attosec_to_autime


class LCAOTDDFT(GPAW):
    def __init__(self, filename=None,
                 propagator=None, fxc=None, **kwargs):
        self.time = 0.0
        self.niter = 0
        self.kick_strength = np.zeros(3)
        self.tddft_initialized = False
        self.action = None
        self.td_hamiltonian = TimeDependentHamiltonian(fxc=fxc)

        self.propagator = propagator
        if filename is None:
            kwargs['mode'] = kwargs.get('mode', 'lcao')
        GPAW.__init__(self, filename, **kwargs)

        # Restarting from a file
        if filename is not None:
            # self.initialize()
            self.set_positions()

    def _write(self, writer, mode):
        GPAW._write(self, writer, mode)
        if self.tddft_initialized:
            w = writer.child('tddft')
            w.write(time=self.time,
                    niter=self.niter,
                    kick_strength=self.kick_strength,
                    propagator=self.propagator.todict())
            self.td_hamiltonian.write(w.child('td_hamiltonian'))

    def read(self, filename):
        reader = GPAW.read(self, filename)
        if 'tddft' in reader:
            r = reader.tddft
            self.time = r.time
            self.niter = r.niter
            self.kick_strength = r.kick_strength
            if self.propagator is None:
                self.propagator = r.propagator
            else:
                self.log('Note! Propagator changed!')
            self.td_hamiltonian.wfs = self.wfs
            self.td_hamiltonian.read(r.td_hamiltonian)

    def tddft_init(self):
        if self.tddft_initialized:
            return

        self.log('-----------------------------------')
        self.log('Initializing time-propagation TDDFT')
        self.log('-----------------------------------')
        self.log()

        assert self.wfs.dtype == complex

        self.timer.start('Initialize TDDFT')

        # Initialize Hamiltonian
        self.td_hamiltonian.initialize(self)

        # Initialize propagator
        self.propagator = create_propagator(self.propagator)
        self.propagator.initialize(self)

        self.log('Propagator:')
        self.log(self.propagator.get_description())
        self.log()

        # Add logger
        TDDFTLogger(self)

        # Call observers before propagation
        self.action = 'init'
        self.call_observers(self.niter)

        self.tddft_initialized = True
        self.timer.stop('Initialize TDDFT')

    def absorption_kick(self, kick_strength):
        self.tddft_init()

        self.timer.start('Kick')

        self.kick_strength = np.array(kick_strength, dtype=float)
        magnitude = np.sqrt(np.sum(self.kick_strength**2))
        direction = self.kick_strength / magnitude

        self.log('----  Applying absorption kick')
        self.log('----  Magnitude: %.8f Hartree/Bohr' % magnitude)
        self.log('----  Direction: %.4f %.4f %.4f' % tuple(direction))

        # Create hamiltonian object for absorption kick
        cef = ConstantElectricField(magnitude * Hartree / Bohr, direction)
        kick_hamiltonian = KickHamiltonian(self, cef)

        # Propagate kick
        self.propagator.kick(kick_hamiltonian, self.time)

        # Call observers after kick
        self.action = 'kick'
        self.call_observers(self.niter)

        self.niter += 1

        self.timer.stop('Kick')

    def propagate(self, time_step=10, iterations=2000):
        self.tddft_init()

        time_step *= attosec_to_autime
        self.maxiter = self.niter + iterations

        self.log('----  About to do %d propagation steps' % iterations)

        self.timer.start('Propagate')
        while self.niter < self.maxiter:
            # Propagate one step
            self.time = self.propagator.propagate(self.time, time_step)

            # Call registered callback functions
            self.action = 'propagate'
            self.call_observers(self.niter)

            self.niter += 1
        self.timer.stop('Propagate')

    def replay(self, **kwargs):
        self.propagator = create_propagator(**kwargs)
        self.tddft_init()
        self.propagator.control_paw(self)



from __future__ import division
from gpaw import GPAW
from gpaw import PW
from gpaw import FermiDirac
from ase.lattice.hexagonal import Graphene
from ase.parallel import parprint

system = Graphene(symbol='C',
                  latticeconstant={'a': 2.467710, 'c': 1.0},
                  size=(1, 1, 1))
system.pbc = (1, 1, 0)
system.center(axis=2, vacuum=4.0)

kpt_refine = {
    'center': [1 / 3, 1 / 3, 0],
    'size': [3, 3, 1],
    'reduce_symmetry': False}
# kpt_refine={"center":[[1./3,1./3,0.],[-1./3,-1./3,0.]], "size":[3,3,1],
#             "reduce_symmetry":True}

calc = GPAW(mode=PW(ecut=400),
            xc=dict(name='PBE', stencil=1),
            kpts={'size': [9, 9, 1], 'gamma': True},
            experimental={'kpt_refine': kpt_refine},
            occupations=FermiDirac(0.026))

system.set_calculator(calc)
energy = system.get_potential_energy()

parprint('Energy', energy)

assert abs(energy - -18.27) < 0.02



import numpy as np

from ase import Atom, Atoms

from gpaw import GPAW, FermiDirac
from gpaw.test import gen
from gpaw.xas import XAS
import gpaw.mpi as mpi

# Generate setup for oxygen with half a core-hole:
gen('Si', name='hch1s', corehole=(1, 0, 0.5))

a = 5.43095
b = a / 2
c = b / 2
d = b + c

si_nonortho = Atoms([Atom('Si', (0, 0, 0)),
                     Atom('Si', (a/4, a/4, a/4))],
                    cell=[(a/2,a/2,0),(a/2,0,a/2),(0,a/2,a/2)],
                    pbc=True)

# calculation with full symmetry
calc = GPAW(nbands=-10,
            h=0.25,
            kpts=(2,2,2),
            occupations=FermiDirac(width=0.05),
            setups={0: 'hch1s'})

si_nonortho.set_calculator(calc)
e = si_nonortho.get_potential_energy()
niter = calc.get_number_of_iterations()
calc.write('si_nonortho_xas_sym.gpw')


# calculation without any symmetry
calc = GPAW(nbands=-10,
            h=0.25,
            kpts=(2,2,2),
            occupations=FermiDirac(width=0.05),
            setups={0: 'hch1s'},
            symmetry='off')

si_nonortho.set_calculator(calc)
e = si_nonortho.get_potential_energy()
niter = calc.get_number_of_iterations()
calc.write('si_nonortho_xas_nosym.gpw')


# restart from file
calc1 = GPAW('si_nonortho_xas_sym.gpw')
calc2 = GPAW('si_nonortho_xas_nosym.gpw')
if mpi.size == 1:
    xas1 = XAS(calc1)
    x, y1 = xas1.get_spectra()
    xas2 = XAS(calc2)
    x2, y2 = xas2.get_spectra(E_in=x)

    assert (np.sum(abs(y1-y2)[0,:500] ** 2 ) < 5e-9)
    assert (np.sum(abs(y1-y2)[1,:500] ** 2 ) < 5e-9)
    assert (np.sum(abs(y1-y2)[2,:500] ** 2 ) < 5e-9)

#else:
#    x = np.linspace(0, 10, 50)
#
#r1 = RecursionMethod(calc1)
#r1.run(40)
#
#r2 = RecursionMethod(calc1)
#r2.run(40)
#
#if mpi.size == 1:
#    y1 = r1.get_spectra(x)
#    y2 = r2.get_spectra(x)
#
#    assert (np.sum(abs(y1-y2)[0,:] ** 2 ) < 5e-9)
#    assert (np.sum(abs(y1-y2)[1,:] ** 2 ) < 5e-9)
#    assert (np.sum(abs(y1-y2)[2,:] ** 2 ) < 5e-9)




from math import factorial as fact
import numpy as np

from gpaw.sphere import lmfact
from gpaw.sphere.legendre import ilegendre, legendre, dlegendre

# Define the Heaviside function
heaviside = lambda x: (1.0+np.sign(x))/2.0

# Define spherical harmoncics and normalization coefficient
C = lambda l,m: (-1)**((m+abs(m))//2)*((2.*l+1.)/(4*np.pi*lmfact(l,m)))**0.5
Y = lambda l,m,theta,phi: C(l,m)*legendre(l,abs(m),np.cos(theta))*np.exp(1j*m*phi)

# Define theta-derivative of spherical harmoncics
dYdtheta = lambda l,m,theta,phi: C(l,m)*dlegendre(l,abs(m),np.cos(theta))*np.exp(1j*m*phi)

# Define phi-derivative of spherical harmoncics
dYdphi = lambda l,m,theta,phi: 1j*m*C(l,m)*legendre(l,abs(m),np.cos(theta))*np.exp(1j*m*phi)

# -------------------------------------------------------------------

def intYY(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /      *
           A    = |  |    Y (u,v) Y (u,v) sin(u) dv du
            LL'   /  /     lm      l'm'
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if l1 = l2 and m1 = m2.
    """
    if (l1,m1) == (l2,m2):
        return C(l1,m1)**2*2*np.pi*ilegendre(l1,m1) # == 1 always, XXX right?
    else:
        return 0.0

# -------------------------------------------------------------------

def intYY_ex(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /      *
           A    = |  |    Y (u,v) sin(u)*cos(v) Y (u,v) sin(u) dv du
            LL'   /  /     lm                    l'm'
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` = 1 and `|m1-m2|` = 1.
    """
    if abs(l1-l2) != 1 or abs(m1-m2) != 1:
        return 0.0
    scale = C(l1,m1)*C(l2,m2)*np.pi
    if abs(m1) == abs(m2)+1:
        return scale*np.sign(l1-l2)/(2.*l2+1.)*ilegendre(l1,m1)
    else:
        assert abs(m1) == abs(m2)-1
        return scale*np.sign(l2-l1)/(2.*l1+1.)*ilegendre(l2,m2)

def intYY_ey(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /      *
           A    = |  |    Y (u,v) sin(u)*sin(v) Y (u,v) sin(u) dv du
            LL'   /  /     lm                    l'm'
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` = 1 and `|m1-m2|` = 1.
    """
    if abs(l1-l2) != 1 or abs(m1-m2) != 1:
        return 0.0
    scale = C(l1,m1)*C(l2,m2)*np.sign(m1-m2)*np.pi/1j
    if abs(m1) == abs(m2)+1:
        return scale*np.sign(l1-l2)/(2.*l2+1.)*ilegendre(l1,m1)
    else:
        assert abs(m1) == abs(m2)-1
        return scale*np.sign(l2-l1)/(2.*l1+1.)*ilegendre(l2,m2)

def intYY_ez(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /      *
           A    = |  |    Y (u,v) cos(u) Y (u,v) sin(u) dv du
            LL'   /  /     lm             l'm'
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` = 1 and m1 = m2.
    """
    if abs(l1-l2) != 1 or m1 != m2:
        return 0.0
    scale = C(l1,m1)*C(l2,m2)*2*np.pi
    return scale*(l2-np.sign(l1-l2)*abs(m1)+heaviside(l1-l2))/(2.*l2+1.)*ilegendre(l1,m1)

# -------------------------------------------------------------------

def intYdYdtheta_ex(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /      *                    d Y(u,v)
           A    = |  |    Y (u,v) cos(u)*cos(v) --- l'm'  sin(u) dv du
            LL'   /  /     lm                    du  
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` is odd and `|m1-m2|` = 1 (stricter rule applies).
    """
    if abs(l1-l2) % 2 != 1 or abs(m1-m2) != 1:
        return 0.0
    scale = -C(l1,m1)*C(l2,m2)*np.pi
    if abs(m1) == abs(m2)+1:
        if l1+1 == l2:
            return scale*2/(2.0*l2+1)*(l2+1)/(2.0*l2-1.0)*fact(l2+abs(m2))/fact(l2-abs(m2)-1)*(l2-abs(m2)-1)
        elif l1-1 == l2:
            return scale*2/(2.0*l2+1)*((l2+1)*fact(l2+abs(m2))/fact(l2-abs(m2))*(l2-abs(m2))-l2/(2.0*l2+3.0)*fact(l2+abs(m2)+1)/fact(l2-abs(m2))*(l2-abs(m2)+1))
        elif l1-l2 > 2: # and (l1-l2)%2 == 1 which is always true
            return -scale*2*abs(m2)*fact(l2+abs(m2))/fact(l2-abs(m2))
        else:
            return 0.0
    else:
        assert abs(m1) == abs(m2)-1
        if l1 == l2+1:
            return -scale*2/(2.0*l1+1.0)*(l1-1)/(2.0*l1-1.0)*fact(l1+abs(m1))/fact(l1-abs(m1)-1)*(l1-abs(m1)-1)
        elif l1 == l2-1:
            return -scale*2/(2.0*l1+1.0)*((l1+1)/(2.0*l1+3.0)*fact(l1+abs(m1)+1)/fact(l1-abs(m1))*(l1-abs(m1)+1)-(abs(m1)+1)*fact(l1+abs(m1))/fact(l1-abs(m1))*(l1-abs(m1)))
        elif l2-l1 > 2: # and (l2-l1)%2 == 1 which is always true
            return scale*2*(abs(m1)+1)*fact(l1+abs(m1))/fact(l1-abs(m1))
        else:
            return 0.0

def intYdYdtheta_ey(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /      *                    d Y(u,v)
           A    = |  |    Y (u,v) cos(u)*sin(v) --- l'm'  sin(u) dv du
            LL'   /  /     lm                    du  
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` is odd and `|m1-m2|` = 1 (stricter rule applies).
    """
    if abs(l1-l2) % 2 != 1 or abs(m1-m2) != 1:
        return 0.0
    scale = -C(l1,m1)*C(l2,m2)*np.sign(m1-m2)*np.pi/1j
    if abs(m1) == abs(m2)+1:
        if l1+1 == l2:
            return scale*2/(2.0*l2+1)*(l2+1)/(2.0*l2-1.0)*fact(l2+abs(m2))/fact(l2-abs(m2)-1)*(l2-abs(m2)-1)
        elif l1-1 == l2:
            return scale*2/(2.0*l2+1)*((l2+1)*fact(l2+abs(m2))/fact(l2-abs(m2))*(l2-abs(m2))-l2/(2.0*l2+3.0)*fact(l2+abs(m2)+1)/fact(l2-abs(m2))*(l2-abs(m2)+1))
        elif l1-l2 > 2: # and (l1-l2)%2 == 1 which is always true
            return -scale*2*abs(m2)*fact(l2+abs(m2))/fact(l2-abs(m2))
        else:
            return 0.0
    else:
        assert abs(m1) == abs(m2)-1
        if l1 == l2+1:
            return -scale*2/(2.0*l1+1.0)*(l1-1)/(2.0*l1-1.0)*fact(l1+abs(m1))/fact(l1-abs(m1)-1)*(l1-abs(m1)-1)
        elif l1 == l2-1:
            return -scale*2/(2.0*l1+1.0)*((l1+1)/(2.0*l1+3.0)*fact(l1+abs(m1)+1)/fact(l1-abs(m1))*(l1-abs(m1)+1)-(abs(m1)+1)*fact(l1+abs(m1))/fact(l1-abs(m1))*(l1-abs(m1)))
        elif l2-l1 > 2: # and (l2-l1)%2 == 1 which is always true
            return scale*2*(abs(m1)+1)*fact(l1+abs(m1))/fact(l1-abs(m1))
        else:
            return 0.0

def intYdYdtheta_ez(l1, m1, l2, m2):
    """Calculates::

                     pi 2pi
                     /  /      *             d Y(u,v)
           A    =  - |  |    Y (u,v) sin(u) --- l'm'  sin(u) dv du
            LL'      /  /     lm             du  
                     0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` = 1 and m1 = m2.
    """
    if abs(l1-l2) != 1 or m1 != m2:
        return 0.0
    scale = -C(l1,m1)*C(l2,m2)*2*np.pi
    return scale*np.sign(l1-l2)*(l2+1-heaviside(l1-l2))*(l2-np.sign(l1-l2)*abs(m1)+heaviside(l1-l2))/(2*l2+1)*ilegendre(l1,m1)

# -------------------------------------------------------------------

def intYdYdphi_ex(l1, m1, l2, m2):
    """Calculates::

                     pi 2pi
                     /  /      *       -1           d Y(u,v)
           A    =  - |  |    Y (u,v) sin(u)*sin(v) --- l'm'  sin(u) dv du
            LL'      /  /     lm                    dv  
                     0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` is odd and `|m1-m2|` = 1 (stricter rule applies).
    """
    if abs(l1-l2) % 2 != 1 or abs(m1-m2) != 1:
        return 0.0
    scale = -C(l1,m1)*C(l2,m2)*np.sign(m1-m2)*np.pi/1j*1j*m2
    if abs(m1) == abs(m2)+1 and l1 > l2: # and (l1-l2)%2 == 1 which is always true
        return scale*2*lmfact(l2,m2)
    elif abs(m1) == abs(m2)-1 and l1 < l2: # and (l2-l1)%2 == 1 which is always true
        return scale*2*lmfact(l1,m1)
    else:
        return 0.0

def intYdYdphi_ey(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /      *       -1           d Y(u,v)
           A    = |  |    Y (u,v) sin(u)*cos(v) --- l'm'  sin(u) dv du
            LL'   /  /     lm                    dv  
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` is odd and `|m1-m2|` = 1 (stricter rule applies).
    """
    if abs(l1-l2) % 2 != 1 or abs(m1-m2) != 1:
        return 0.0
    scale = C(l1,m1)*C(l2,m2)*np.pi*1j*m2
    if abs(m1) == abs(m2)+1 and l1 > l2: # and (l1-l2)%2 == 1 which is always true
        return scale*2*lmfact(l2,m2)
    elif abs(m1) == abs(m2)-1 and l1 < l2: # and (l2-l1)%2 == 1 which is always true
        return scale*2*lmfact(l1,m1)
    else:
        return 0.0

def intYdYdphi_ez(l1, m1, l2, m2): #XXX this is silly
    return 0.0




from ase.build import bulk
from gpaw import GPAW
from gpaw.mpi import world

# Run single SCF iteration and compare total energy with elpa vs. scalapack

energies = []

for elpasolver in [None, '1stage', '2stage']:
    atoms = bulk('Al')
    calc = GPAW(mode='lcao', basis='sz(dzp)',
                kpts=[2, 2, 2],
                parallel=dict(sl_auto=True, use_elpa=elpasolver is not None,
                              band=2 if world.size > 4 else 1,
                              kpt=2 if world.size > 2 else 1,
                              elpasolver=elpasolver),
                txt='-')

    def stopcalc():
        calc.scf.converged = True

    calc.attach(stopcalc, 2)
    atoms.calc = calc
    E = atoms.get_potential_energy()
    energies.append(E)

    err = abs(E - energies[0])
    assert err < 1e-10, ' '.join(['err', str(err), 'energies:', str(energies)])



from __future__ import print_function
from ase import Atom, Atoms
from gpaw import GPAW
from gpaw.test import equal

a = 5.0
d = 1.0
x = d / 3**0.5
atoms = Atoms([Atom('C', (0.0, 0.0, 0.0)),
                     Atom('H', (x, x, x)),
                     Atom('H', (-x, -x, x)),
                     Atom('H', (x, -x, -x)),
                     Atom('H', (-x, x, -x))],
                    cell=(a, a, a),
                    pbc=False)

atoms.positions[:] += a / 2
calc = GPAW(h=0.25, nbands=4, convergence={'eigenstates': 7.8e-10})
atoms.calc = calc
energy = atoms.get_potential_energy()
niter = calc.get_number_of_iterations()

# The three eigenvalues e[1], e[2], and e[3] must be degenerate:
e = calc.get_eigenvalues()
print(e[1] - e[3])
equal(e[1], e[3], 9.3e-8)

energy_tolerance = 0.002
niter_tolerance = 0
equal(energy, -23.631, energy_tolerance)

# Calculate non-selfconsistent PBE eigenvalues:
from gpaw.xc.tools import vxc
epbe0 = e - vxc(calc)[0, 0] + vxc(calc, 'PBE')[0, 0]

# Calculate selfconsistent PBE eigenvalues:
calc.set(xc='PBE')
energy = atoms.get_potential_energy()
epbe = calc.get_eigenvalues()

de = epbe[1] - epbe[0]
de0 = epbe0[1] - epbe0[0]
print(de, de0)
equal(de, de0, 0.001)



from __future__ import print_function
import sys
from numpy import test

from gpaw.mpi import rank

_stdout = sys.stdout
_stderr = sys.stderr
# numpy tests write to stderr
sys.stderr = open("numpy_test%02d.out" % rank, "w")
result = test(verbose=10)
sys.stdout = _stdout
sys.stderr = _stderr
if not result.wasSuccessful():
    print("numpy_test%02d.out" % rank, result.errors, result.failures, file=sys.stderr)
assert result.wasSuccessful()




from ase import Atoms
from gpaw import GPAW


class ObsTest:
    def __init__(self):
        self.steps = []

    def run(self):
        self.steps.append(calc.iter)
        
    def converge(self):
        calc.scf.converged = True
            

H = Atoms('H', positions=[(1.5, 1.5, 1.5)])
H.set_cell((3, 3, 3))

calc = GPAW(convergence={'density': -1}, maxiter=11)
            
AllCall = ObsTest()
EvenCall = ObsTest()
OneCall = ObsTest()
FinalCall = ObsTest()

calc.attach(AllCall.run, 1)
calc.attach(EvenCall.run, 2)
calc.attach(OneCall.run, -7)
calc.attach(OneCall.converge, -7)
calc.attach(FinalCall.run, 0)

H.set_calculator(calc)
H.get_potential_energy()
    
assert AllCall.steps == [1, 2, 3, 4, 5, 6, 7]
assert EvenCall.steps == [2, 4, 6, 7]
assert OneCall.steps == [7]
assert FinalCall.steps == [7]



import gpaw.mpi


class LrCommunicators:
    def __init__(self, world=None, dd_size=None, eh_size=None):
        """Create communicators for LrTDDFT calculation.
        
        Input parameters:
        
        world
          MPI parent communicator (usually gpaw.mpi.world)
        
        dd_size
          Over how many processes is domain distributed.
      
        eh_size
          Over how many processes are electron-hole pairs distributed.
      
        Note
        ----
        Sizes must match, i.e., world.size must be equal to
        dd_size x eh_size, e.g., 1024 = 64*16

        Tip
        ---
        Use enough processes for domain decomposition (dd_size) to fit
        everything (easily) into memory, and use the remaining processes
        for electron-hole pairs as K-matrix build is trivially parallel
        over them.


        Pass lr_comms.dd_comm to ground state calc when reading for LrTDDFT.


        ----------------------------------------------------------------------

        Example (for 8 MPI processes)::

          lr_comms = LrCommunicators(gpaw.mpi.world, 4, 2)
          txt = 'lr_%06d_%06d.txt' % (lr_comms.dd_comm.rank,
                                      lr_comms.eh_comm.rank)
          lr = LrTDDFTindexed(GPAW('unocc.gpw', communicator=lr_comms.dd_comm),
                              lr_communicators=lr_comms, txt=txt)
        
        """

        self.parent_comm = None
        self.dd_comm = None
        self.eh_comm = None
        
        self.world = world
        self.dd_size = dd_size
        self.eh_size = eh_size
        
        if self.world is None:
            return
        if self.dd_size is None:
            return
        
        if self.eh_size is None:
            self.eh_size = self.world.size // self.dd_size
            
        self.parent_comm = self.world
        
        if self.world.size != self.dd_size * self.eh_size:
            raise RuntimeError('Domain decomposition processes (dd_size) '
                               'times electron-hole (eh_size) processes '
                               'does not match with total processes '
                               '(world size != dd_size * eh_size)')
            
        dd_ranks = []
        eh_ranks = []
        for k in range(self.world.size):
            if k // self.dd_size == self.world.rank // self.dd_size:
                dd_ranks.append(k)
            if k % self.dd_size == self.world.rank % self.dd_size:
                eh_ranks.append(k)
        self.dd_comm = self.world.new_communicator(dd_ranks)
        self.eh_comm = self.world.new_communicator(eh_ranks)

    def initialize(self, calc):
        if self.parent_comm is None:
            if calc is not None:
                self.dd_comm = calc.density.gd.comm
                self.parent_comm = self.dd_comm.parent
                if self.parent_comm.size != self.dd_comm.size:
                    raise RuntimeError('Invalid communicators in LrTDDFT2. Ground state calculator domain decomposition communicator and its parent (or actually its parent parent) has different size. Please set up LrCommunicators explicitly to avoid this. Or contact developers if this is intentional.')
                self.eh_comm = gpaw.mpi.serial_comm
            else:
                self.parent_comm = gpaw.mpi.serial_comm
                self.dd_comm = gpaw.mpi.serial_comm
                self.eh_comm = gpaw.mpi.serial_comm
        else:
            # Check that parent_comm is valid
            if self.parent_comm != self.eh_comm.parent:
                raise RuntimeError(
                    'Invalid communicators in LrTDDFT2. LrTDDFT2 parent '
                    'communicator does is not parent of electron-hole '
                    'communicator. Please set up LrCommunicators explicitly '
                    'to avoid this.')
            if self.parent_comm != self.dd_comm.parent:
                raise RuntimeError(
                    'Invalid communicators in LrTDDFT2. LrTDDFT2 parent '
                    'communicator does is not parent of domain decomposition '
                    'communicator. Please set up LrCommunicators explicitly '
                    'to avoid this.')

    # Do not use so slow... unless absolutely necessary
    # def index_of_kss(self,i,p):
    #     for (ind,kss) in enumerate(self.kss_list):
    #         if kss.occ_ind == i and kss.unocc_ind == p:
    #             return ind
    #     return None

    def get_local_eh_index(self, ip):
        if ip % self.eh_comm.size != self.eh_comm.rank:
            return None
        return ip // self.eh_comm.size

    def get_local_dd_index(self, jq):
        if jq % self.dd_comm.size != self.dd_comm.rank:
            return None
        return jq // self.dd_comm.size

    def get_global_eh_index(self, lip):
        return lip * self.eh_comm.size + self.eh_comm.rank

    def get_global_dd_index(self, ljq):
        return ljq * self.dd_comm.size + self.dd_comm.rank

    def get_matrix_elem_proc_and_index(self, ip, jq):
        ehproc = ip % self.eh_comm.size
        ddproc = jq % self.dd_comm.size
        proc = ehproc * self.dd_comm.size + ddproc
        lip = ip // self.eh_comm.size
        ljq = jq // self.eh_comm.size
        return (proc, ehproc, ddproc, lip, ljq)



"""A proton in an electric field."""
from ase import Atoms
from ase.units import Hartree, Bohr

from gpaw import GPAW
from gpaw.external import ConstantElectricField

h = Atoms('H')
h.center(vacuum=2.5)
h.calc = GPAW(external=ConstantElectricField(1.0),  # 1 V / Ang
              charge=1,
              txt='h.txt')
e = h.get_potential_energy()
f1 = h.get_forces()[0, 2]
h[0].z += 0.001
de = h.get_potential_energy() - e
f2 = -de / 0.001
print(f1, f2)
assert abs(f1 - 1) < 1e-4
assert abs(f2 - 1) < 5e-3

# Check writing and reading:
h.calc.write('h')
vext = GPAW('h', txt=None).hamiltonian.vext
assert abs(vext.field_v[2] - 1.0 * Bohr / Hartree) < 1e-13



from __future__ import print_function
from math import pi, sqrt
import numpy as np
from gpaw.utilities.tools import coordinates
from gpaw.utilities.gauss import Gaussian
from gpaw.grid_descriptor import GridDescriptor
from gpaw.test import equal
from gpaw.poisson import PoissonSolver

def norm(a):
    return np.sqrt(np.sum(a.ravel()**2)) / len(a.ravel())

# Initialize classes
a = 20 # Size of cell
N = 48 # Number of grid points
Nc = (N, N, N)                # Number of grid points along each axis
gd = GridDescriptor(Nc, (a,a,a), 0)    # Grid-descriptor object
solver = PoissonSolver(nn=3)  # Numerical poisson solver
solver.set_grid_descriptor(gd)
solve = solver.solve
xyz, r2 = coordinates(gd)     # Matrix with the square of the radial coordinate
print(r2.shape)
r  = np.sqrt(r2)             # Matrix with the values of the radial coordinate
nH = np.exp(-2 * r) / pi     # Density of the hydrogen atom
gauss = Gaussian(gd)          # An instance of Gaussian

# /------------------------------------------------\
# | Check if Gaussian densities are made correctly |
# \------------------------------------------------/
for gL in range(2, 9):
    g = gauss.get_gauss(gL) # a gaussian of gL'th order
    print('\nGaussian of order', gL)
    for mL in range(9):
        m = gauss.get_moment(g, mL) # the mL'th moment of g
        print('  %s\'th moment = %2.6f' % (mL, m))
        equal(m, gL == mL, 1e-4)

# Check the moments of the constructed 1s density
print('\nDensity of Hydrogen atom')
for L in range(4):
    m = gauss.get_moment(nH, L)
    print('  %s\'th moment = %2.6f' % (L, m))
    equal(m, (L == 0) / sqrt(4 * pi), 1.5e-3)

# Check that it is removed correctly
v = gauss.remove_moment(nH, 0)
m = gauss.get_moment(nH, 0)
print('\nZero\'th moment of compensated Hydrogen density =', m)
equal(m, 0., 1e-7)


# /-------------------------------------------------\
# | Check if Gaussian potentials are made correctly |
# \-------------------------------------------------/

# Array for storing the potential
pot = gd.zeros(dtype=float, global_array=False)
for L in range(7): # Angular index of gaussian
    # Get analytic functions
    ng = gauss.get_gauss(L)
    vg = gauss.get_gauss_pot(L)

    # Solve potential numerically
    niter = solve(pot, ng, charge=None, zero_initial_phi=True)

    # Determine residual
    residual = norm(pot - vg)
    residual = gd.integrate((pot - vg)**2)**0.5

    # print result
    print('L=%s, processor %s of %s: %s' % (
        L,
        gd.comm.rank + 1,
        gd.comm.size,
        residual))

    assert residual < 0.6

# mpirun -np 2 python gauss_func.py --gpaw-parallel --gpaw-debug



from gpaw.analyse.vdwradii import vdWradii

# data from A. Bondi, J. Phys. Chem. 68 (1964) 441
data_Bondi = { # units Anstrom
    'He' : 1.40,
    'Ne' : 1.54,
    'Ar' : 1.88,
    'Kr' : 2.02,
    'Xe' : 2.16
}
# data from Felix Hanke (FHI-AIMS ?)
data_Hanke = { # units Anstrom
    'H' : 1.640449351,
    'C' : 1.8997461838999998,
    'N' : 1.7674518813999998,
    'O' : 1.6880752998999997,
    'Cu': 1.9897063095999996,
}    
for symbol in ['He', 'Ne', 'Ar', 'Kr', 'H', 'C', 'N', 'O', 'Cu']:
    R = vdWradii([symbol], 'PBE')[0]
    if symbol in data_Bondi:
        Rref = data_Bondi[symbol]
    else:
        Rref = data_Hanke[symbol]
    error = abs(R - Rref)
    print("symbol, R, Rref, error:", symbol, R, Rref, error)
    assert(error < 0.05)



from distutils.version import LooseVersion

from ase.phonons import Phonons
from ase import Atoms, __version__
import numpy as np

from gpaw import GPAW
from gpaw.elph.electronphonon import ElectronPhononCoupling

if LooseVersion(__version__) < '3.18':
    from unittest import SkipTest
    raise SkipTest


a = 0.90
atoms = Atoms('H',
              cell=np.diag([a, 2.1, 2.1]),
              positions=[[0, 0, 0]],
              pbc=(1, 0, 0))

atoms.center()
supercell = (2, 1, 1)
parameters = {'mode': 'lcao',
              'kpts': {'size': (2, 1, 1), 'gamma': True},
              'txt': None,
              'basis': 'dzp',
              'symmetry': {'point_group': False},
              'xc': 'PBE'}
elph_calc = GPAW(**parameters)
atoms.set_calculator(elph_calc)
atoms.get_potential_energy()
gamma_bands = elph_calc.wfs.kpt_u[0].C_nM

elph = ElectronPhononCoupling(atoms, elph_calc, supercell=supercell,
                              calculate_forces=True)
elph.run()

parameters['parallel'] = {'domain': 1}
elph_calc = GPAW(**parameters)
elph = ElectronPhononCoupling(atoms, calc=None, supercell=supercell)
elph.set_lcao_calculator(elph_calc)
elph.calculate_supercell_matrix(dump=1)

ph = Phonons(atoms=atoms, name='phonons', supercell=supercell, calc=None)
ph.read()
kpts = [[0, 0, 0]]
frequencies, modes = ph.band_structure(kpts, modes=True)

c_kn = np.array([[gamma_bands[0]]])
g_qklnn = elph.bloch_matrix(c_kn=c_kn, kpts=kpts, qpts=kpts, u_ql=modes)



from ase import Atoms
from gpaw import GPAW
from gpaw.spinorbit import get_spinorbit_eigenvalues
from gpaw.test import equal

a = Atoms('Kr')
a.center(vacuum=3.0)

calc = GPAW(mode='pw', xc='LDA')

a.calc = calc
a.get_potential_energy()

e_n = calc.get_eigenvalues()
e_m = get_spinorbit_eigenvalues(calc)

equal(e_n[0] - e_m[0], 0.0, 1.0e-3)
equal(e_n[1] - e_m[2], 0.452, 1.0e-3)
equal(e_n[2] - e_m[4], -0.226, 1.0e-3)



from __future__ import print_function
from ase.build import molecule
from gpaw import GPAW
from gpaw.poisson import FDPoissonSolver as PoissonSolver
from gpaw.atom.basis import BasisMaker
from gpaw.test import equal

# Tests basis set super position error correction

# Compares a single hydrogen atom to a system of one hydrogen atom
# and one ghost hydrogen atom.  The systems should have identical properties,
# i.e. the ghost orbital should have a coefficient of 0.

b = BasisMaker('H').generate(1, 0, energysplit=0.005)

system = molecule('H2')
system.center(vacuum=6.0)


def prepare(setups):
    calc = GPAW(basis={'H': b}, mode='lcao',
                setups=setups, h=0.2,
                poissonsolver=PoissonSolver(nn='M', relax='GS', eps=1e-5),
                spinpol=False,
                nbands=1)
    system.set_calculator(calc)
    return calc


calc = prepare({0: 'paw', 1: 'ghost'})
system.set_calculator(calc)
e_bsse = system.get_potential_energy()
niter_bsse = calc.get_number_of_iterations()

c_nM = calc.wfs.kpt_u[0].C_nM
print('coefs')
print(c_nM)
print('energy', e_bsse)

# Reference system which is just a hydrogen
sys0 = system[0:1].copy()
calc = prepare('paw')
sys0.set_calculator(calc)
e0 = sys0.get_potential_energy()
niter0 = calc.get_number_of_iterations()
print('e0, e_bsse = ', e0, e_bsse)

# One coefficient should be very small (0.012), the other very large (0.99)
assert abs(1.0 - abs(c_nM[0, 0])) < 0.02
assert abs(c_nM[0, 1]) < 0.02
assert abs(e_bsse - e0) < 2e-3

energy_tolerance = 0.002
niter_tolerance = 3
equal(e_bsse, 0.02914, energy_tolerance)  # svnversion 5252
equal(niter_bsse, 7, niter_tolerance)  # svnversion 5252
equal(e0, 0.03038, energy_tolerance)  # svnversion 5252
equal(niter0, 6, niter_tolerance)  # svnversion 5252



"""
Python unit testing framework, based on Erich Gamma's JUnit and Kent Beck's
Smalltalk testing framework. Parallel support by Christian Glinsvad.

For further information please consult the documentation for the Unittest
module, from which this module is derived. Online reference manual at:

  http://docs.python.org/dev/library/unittest.html

Copyright (c) 1999-2003 Steve Purcell
This module is free software, and you may redistribute it and/or modify
it under the same terms as Python itself, so long as this copyright message
and disclaimer are retained in their original form.

IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF
THIS CODE, EVEN IF THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.

THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE.  THE CODE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
"""
from __future__ import print_function
import sys
import numpy as np
from unittest import TestResult, TestCase, TestSuite, \
    _TextTestResult, TextTestRunner, TestLoader, \
    FunctionTestCase, TestProgram

from ase.utils import devnull

from gpaw import debug
from gpaw.mpi import world, broadcast_string

# -------------------------------------------------------------------
# Exported classes and functions
# -------------------------------------------------------------------

__all__ = ['ParallelTestResult', 'ParallelTestCase', 'ParallelTestSuite',
           'ParallelTextTestRunner', 'ParallelTestLoader',
           'ParallelFunctionTestCase', 'main', 'defaultParallelTestLoader']

# -------------------------------------------------------------------
# Limited backward compatibility
# -------------------------------------------------------------------

if sys.version_info[:2] < (2, 3):
    raise RuntimeError('Python 2.3 or greater required!')

try:
    from unittest import __version__
except ImportError:
    unittest_version = (2, 0)
else:
    unittest_version = tuple(map(int, __version__.split('.')))

# User interface should at least comply with Unittest version 1.56 rev. 34209
if unittest_version < (1,56):
    _TestCase = TestCase

    class TestCase(_TestCase):
        assertTrue = _TestCase.assertTrue
        assertFalse = _TestCase.assertFalse

# -------------------------------------------------------------------
# Test framework core
# -------------------------------------------------------------------

class ParallelTestResult(TestResult):
    __doc__ = TestResult.__doc__

    def __init__(self, comm=None):
        if comm is None:
            comm = world
        self.comm = comm
        self.outcomes = []
        self.last_errors = np.empty(self.comm.size, dtype=bool)
        self.last_failed = np.empty(self.comm.size, dtype=bool)
        TestResult.__init__(self)

    def startTest(self, test):
        "Called when the given test is about to be run. Resets global flags."
        self.last_errors.fill(False)
        self.last_failed.fill(False)
        TestResult.startTest(self, test)

    def stopTest(self, test):
        """Called when the given test has been run. If the stop flag was
        raised beforehand, will broadcast to raise flags for global stop."""
        stop_flags = np.empty(self.comm.size, dtype=bool)
        self.comm.all_gather(np.array([self.shouldStop]), stop_flags)
        self.shouldStop = stop_flags.any()
        TestResult.stopTest(self, test)

    def _exc_info_to_string(self, err, test):
        # Includes second argument as of Unittest version 1.63 rev. 34859
        if unittest_version < (1,63):
            return TestResult._exc_info_to_string(self, err)
        else:
            return TestResult._exc_info_to_string(self, err, test)

    def addStatus(self, test, err=None, error=False, failure=False):
        """Negotiate global status immediately after a test has been run.
        Called on all processor with the local test status (i.e. error, failure
        or success). 'err' is a tuple of values as returned by sys.exc_info().
        """
        if error and failure:
            raise RuntimeError('Parallel unittest can\'t handle simultaneous' \
                               + ' errors and failures within a single test.')

        self.comm.all_gather(np.array([error]), self.last_errors)
        if self.last_errors.any():
            all_texts = []
            for rank in np.argwhere(self.last_errors).ravel():
                if rank == self.comm.rank:
                    assert self.last_errors[self.comm.rank]
                    text = self._exc_info_to_string(err, test)
                else:
                    text = None
                text = broadcast_string(text, root=rank, comm=self.comm)
                all_texts.append((rank,text))
            self.errors.append((test, all_texts))

        self.comm.all_gather(np.array([failure]), self.last_failed)
        if self.last_failed.any():
            all_texts = []
            for rank in np.argwhere(self.last_failed).ravel():
                if rank == self.comm.rank:
                    assert self.last_failed[self.comm.rank]
                    text = self._exc_info_to_string(err, test)
                else:
                    text = None
                text = broadcast_string(text, root=rank, comm=self.comm)
                all_texts.append((rank,text))
            self.failures.append((test, all_texts))

    def addError(self, test, err):
        """Negotiate global status. Called when an error has occurred on a
        processor. 'err' is a tuple of values as returned by sys.exc_info().
        """
        self.addStatus(test, err, error=True)

    def addFailure(self, test, err):
        """Negotiate global status. Called when an failure has occurred on a
        processor. 'err' is a tuple of values as returned by sys.exc_info().
        """
        self.addStatus(test, err, failure=True)

    def addSuccess(self, test):
        """Probe global status for potential errors by passive negotiation.
        Called when a test has completed successfully on a processor.
        """
        self.addStatus(test)

    def addSkip(self, test, reason):
        """Called when a test is skipped. Not ready!"""
        raise NotImplementedError

    def addExpectedFailure(self, test, err):
        """Called when an expected failure/error occurred. Not ready!"""
        raise NotImplementedError

    def addUnexpectedSuccess(self, test):
        """Called when a test was expected to fail, but succeed. Not ready!"""
        raise NotImplementedError


class ParallelTestCase(TestCase):
    __doc__ = TestCase.__doc__

    def defaultTestResult(self):
        return ParallelTestResult()

    def debug(self):
        """Run the test without collecting errors in a TestResult"""
        print('WARNING: Test case is strictly serial in debug mode!')
        TestCase.debug(self)


# No point in implementing these as the originals work just fine
ParallelTestSuite = TestSuite
ParallelFunctionTestCase = FunctionTestCase


# -------------------------------------------------------------------
# Locating and loading tests
# -------------------------------------------------------------------

# No point in implementing these as the originals work just fine
ParallelTestLoader = TestLoader
defaultParallelTestLoader = ParallelTestLoader()


# -------------------------------------------------------------------
# Text UI
# -------------------------------------------------------------------

class _ParallelTextTestResult(ParallelTestResult, _TextTestResult):
    __doc__ = _TextTestResult.__doc__
    allErrors = False

    def __init__(self, comm, *args, **kwargs):
        ParallelTestResult.__init__(self, comm)
        _TextTestResult.__init__(self, *args, **kwargs) # use new-style?

    def startTest(self, test):
        ParallelTestResult.startTest(self, test)
        if self.showAll:
            self.stream.write(self.getDescription(test))
            self.stream.write(" ... ")
            self.stream.flush()

    def addSuccess(self, test):
        ParallelTestResult.addSuccess(self, test)

    def addError(self, test, err):
        ParallelTestResult.addError(self, test, err)

    def addFailure(self, test, err):
        ParallelTestResult.addFailure(self, test, err)

    def shortRanks(self, ranks):
        if len(ranks) == 0:
            return 'none'
        elif isinstance(ranks, np.ndarray) and ranks.dtype == bool:
            if ranks.all():
                return 'all'
            ranks = np.argwhere(ranks).ravel()
        ranks = np.sort(ranks)
        if np.all(ranks == range(self.comm.size)):
            return 'all'
        return 'rank ' + ','.join(map(str,ranks))

    def stopTest(self, test):
        self.stream.flush()
        self.comm.barrier()

        if self.last_errors.any() and self.last_failed.any():
            if not debug:
                raise RuntimeError('Parallel unittest can\'t handle ' \
                    'simultaneous errors and failures within a single test.')
            if self.showAll:
                error_rankinfo = self.shortRanks(self.last_errors)
                fail_rankinfo = self.shortRanks(self.last_failed)
                self.stream.writeln("ERROR (%s) / FAIL (%s)" \
                                    % (error_rankinfo, fail_rankinfo))
            elif self.dots:
                self.stream.writeln('M')
        elif self.last_errors.any():
            if self.showAll:
                rankinfo = self.shortRanks(self.last_errors)
                self.stream.writeln("ERROR (%s)" % rankinfo)
            elif self.dots:
                self.stream.writeln('E')
        elif self.last_failed.any():
            if self.showAll:
                rankinfo = self.shortRanks(self.last_failed)
                self.stream.writeln("FAIL (%s)" % rankinfo)
            elif self.dots:
                self.stream.writeln('F')
        else:
            if self.showAll:
                self.stream.writeln("ok")
            else:
                self.stream.writeln('.')

    def printErrorList(self, flavour, errors):
        for test, all_texts in errors:
            description = self.getDescription(test)
            if all_texts:
                self.stream.writeln(self.separator1)
            if self.allErrors:
                for rank, text2 in all_texts:
                    rankinfo = self.shortRanks([rank]).upper()
                    text1 = '%s-%s: %s' % (flavour, rankinfo, description)
                    self.stream.writeln(text1)
                    self.stream.writeln(self.separator2)
                    self.stream.writeln(text2)
            else:
                text_ranks = {}
                for rank, text2 in all_texts:
                    if text2 not in text_ranks:
                        text_ranks[text2] = []
                    text_ranks[text2].append(rank)
                for text2, ranks in text_ranks.items():
                    rankinfo = self.shortRanks(ranks).upper()
                    text1 = '%s-%s: %s' % (flavour, rankinfo, description)
                    self.stream.writeln(text1)
                    self.stream.writeln(self.separator2)
                    self.stream.writeln(text2)


class ParallelTextTestRunner(TextTestRunner):
    __doc__ = TextTestRunner.__doc__
    logfile = None

    def __init__(self, comm=None, stream=sys.stderr, **kwargs):
        if comm is None:
            comm = world
        self.comm = comm
        if self.comm.rank != 0:
            stream = devnull
        elif isinstance(stream, str):
            self.logfile = stream
            stream = open(self.logfile, 'w', buffering=0)
        TextTestRunner.__init__(self, stream=stream, **kwargs)

    def _makeResult(self):
        return _ParallelTextTestResult(self.comm, self.stream, \
            self.descriptions, self.verbosity)

    def run(self, test):
        stderr_old = sys.stderr
        try:
            sys.stderr = self.stream
            testresult = TextTestRunner.run(self, test)
        finally:
            sys.stderr = stderr_old
        return testresult


# -------------------------------------------------------------------
# Facilities for running tests from the command line
# -------------------------------------------------------------------

class ParallelTestProgram(TestProgram):
    __doc__ = TestProgram.__doc__

    def runTests(self):
        if self.testRunner is None:
            self.testRunner = ParallelTextTestRunner(verbosity=self.verbosity)
        TestProgram.runTests(self)

main = ParallelTestProgram




from ase import Atoms
from gpaw import GPAW
from gpaw.xc.vdw import VDWFunctional

vdw = VDWFunctional('vdW-DF', verbose=1)
L = 2.5
a = Atoms('H', cell=(L, L, L), pbc=True, calculator=GPAW(nbands=1))
e = a.get_potential_energy()
e2 = a.calc.get_xc_difference(vdw)
print(e, e2)
assert (vdw.shape == (24, 24, 24)).all()



from __future__ import division, print_function

import functools
import itertools
import os
import pickle
import warnings
from math import pi

import numpy as np
from ase.dft.kpoints import monkhorst_pack
from ase.units import Ha
from ase.utils import opencew, devnull, pickleload
from ase.utils.timing import timer
from ase.parallel import paropen

from gpaw import GPAW
import gpaw.mpi as mpi
from gpaw import debug
from gpaw.kpt_descriptor import KPointDescriptor
from gpaw.response.chi0 import Chi0, HilbertTransform
from gpaw.response.pair import PairDensity
from gpaw.response.wstc import WignerSeitzTruncatedCoulomb
from gpaw.response.kernels import get_coulomb_kernel
from gpaw.response.kernels import get_integrated_kernel
from gpaw.response.fxckernel_calc import calculate_kernel
from gpaw.xc.fxc import set_flags
from gpaw.wavefunctions.pw import (PWDescriptor, count_reciprocal_vectors,
                                   PWMapping)
from gpaw.xc.exx import EXX, select_kpts
from gpaw.xc.tools import vxc
from gpaw.utilities.progressbar import ProgressBar


class G0W0(PairDensity):
    def __init__(self, calc, filename='gw', restartfile=None,
                 kpts=None, bands=None, relbands=None, nbands=None, ppa=False,
                 xc='RPA', fxc_mode='GW', density_cut=1.e-6, do_GW_too=False,
                 av_scheme=None, Eg=None,
                 truncation=None, integrate_gamma=0,
                 ecut=150.0, eta=0.1, E0=1.0 * Ha,
                 domega0=0.025, omega2=10.0, q0_correction=False,
                 anisotropy_correction=None,
                 nblocks=1, savew=False, savepckl=True,
                 maxiter=1, method='G0W0', mixing=0.2,
                 world=mpi.world, ecut_extrapolation=False,
                 nblocksmax=False, gate_voltage=None):

        """G0W0 calculator.

        The G0W0 calculator is used is used to calculate the quasi
        particle energies through the G0W0 approximation for a number
        of states.

        calc: str or PAW object
            GPAW calculator object or filename of saved calculator object.
        filename: str
            Base filename of output files.
        restartfile: str
            File that stores data necessary to restart a calculation.
        kpts: list
            List of indices of the IBZ k-points to calculate the quasi particle
            energies for.
        bands: tuple of two ints
            Range of band indices, like (n1, n2), to calculate the quasi
            particle energies for. Bands n where n1<=n<n2 will be
            calculated.  Note that the second band index is not included.
        relbands: tuple of two ints
            Same as *bands* except that the numbers are relative to the
            number of occupied bands.
            E.g. (-1, 1) will use HOMO+LUMO.
        ecut: float
            Plane wave cut-off energy in eV.
        ecut_extrapolation: bool or array
            If set to True an automatic extrapolation of the selfenergy to
            infinite cutoff will be performed based on three points
            for the cutoff energy.
            If an array is given, the extrapolation will be performed based on
            the cutoff energies given in the array.
        nbands: int
            Number of bands to use in the calculation. If None, the number will
            be determined from :ecut: to yield a number close to the number of
            plane waves used.
        ppa: bool
            Sets whether the Godby-Needs plasmon-pole approximation for the
            dielectric function should be used.
        xc: str
            Kernel to use when including vertex corrections.
        fxc_mode: str
            Where to include the vertex corrections; polarizability and/or
            self-energy. 'GWP': Polarizability only, 'GWS': Self-energy only,
            'GWG': Both.
        density_cut: float
            Cutoff for density when constructing kernel.
        do_GW_too: bool
            When carrying out a calculation including vertex corrections, it
            is possible to get the standard GW results at the same time
            (almost for free).
        av_scheme: str
            'wavevector'. Method to construct kernel. Only
            'wavevector' has been tested and works here. The implementation
            could be extended to include the 'density' method which has been
            tested for total energy calculations (rALDA etc.)
        Eg: float
            Gap to apply in the 'JGMs' (simplified jellium-with-gap) kernel.
            If None the DFT gap is used.
        truncation: str
            Coulomb truncation scheme. Can be either wigner-seitz,
            2D, 1D, or 0D
        integrate_gamma: int
            Method to integrate the Coulomb interaction. 1 is a numerical
            integration at all q-points with G=[0,0,0] - this breaks the
            symmetry slightly. 0 is analytical integration at q=[0,0,0] only -
            this conserves the symmetry. integrate_gamma=2 is the same as 1,
            but the average is only carried out in the non-periodic directions.
        E0: float
            Energy (in eV) used for fitting in the plasmon-pole approximation.
        domega0: float
            Minimum frequency step (in eV) used in the generation of the non-
            linear frequency grid.
        omega2: float
            Control parameter for the non-linear frequency grid, equal to the
            frequency where the grid spacing has doubled in size.
        gate_voltage: float
            Shift Fermi level of ground state calculation by the
            specified amount.
        q0_correction: bool
            Analytic correction to the q=0 contribution applicable to 2D
            systems.
        anisotropy_correction: bool
            Old term for the q0_correction.
        nblocks: int
            Number of blocks chi0 should be distributed in so each core
            does not have to store the entire matrix. This is to reduce
            memory requirement. nblocks must be less than or equal to the
            number of processors.
        nblocksmax: bool
            Cuts chi0 into as many blocks as possible to reduce memory
            requirements as much as possible.
        savew: bool
            Save W to a file.
        savepckl: bool
            Save output to a pckl file.
        method: str
            G0W0 or GW0(eigenvalue selfconsistency in G) currently available.
        maxiter: int
            Number of iterations in a GW0 calculation.
        mixing: float
            Number between 0 and 1 determining how much of previous
            iteration's eigenvalues to mix with.
        ecut_extrapolation: bool
            Carries out the extrapolation to infinite cutoff automatically.
        """

        if world.rank != 0:
            txt = devnull
        else:
            txt = open(filename + '.txt', 'w')

        p = functools.partial(print, file=txt)
        p('  ___  _ _ _ ')
        p(' |   || | | |')
        p(' | | || | | |')
        p(' |__ ||_____|')
        p(' |___|')
        p()

        self.inputcalc = calc

        if ppa and (nblocks > 1 or nblocksmax):
            p('PPA is currently not compatible with block parallellisation. '
              'Setting nblocks=1 and continuing.')
            nblocks = 1
            nblocksmax = False

        if ecut_extrapolation is True:
            pct = 0.8
            necuts = 3
            ecut_e = ecut * (1 + (1. / pct - 1) * np.arange(necuts)[::-1] /
                             (necuts - 1))**(-2 / 3)
            """It doesn't make sence to save W in this case since
            W is calculated for different cutoffs:"""
            assert not savew
        elif isinstance(ecut_extrapolation, (list, np.ndarray)):
            ecut_e = np.array(np.sort(ecut_extrapolation))
            ecut = ecut_e[-1]
            assert not savew
        else:
            ecut_e = np.array([ecut])

        # Check if nblocks is compatible, adjust if not
        if nblocksmax:
            nblocks = world.size
            if isinstance(calc, GPAW):
                raise Exception('Using a calulator is not implemented at '
                                'the moment, load from file!')
                # nblocks_calc = calc
            else:
                nblocks_calc = GPAW(calc, txt=None)
            ngmax = []
            for q_c in nblocks_calc.wfs.kd.bzk_kc:
                qd = KPointDescriptor([q_c])
                pd = PWDescriptor(np.min(ecut) / Ha,
                                  nblocks_calc.wfs.gd, complex, qd)
                ngmax.append(pd.ngmax)
            nG = np.min(ngmax)

            while nblocks > nG**0.5 + 1 or world.size % nblocks != 0:
                nblocks -= 1

            mynG = (nG + nblocks - 1) // nblocks
            assert mynG * (nblocks - 1) < nG

        self.ecut_e = ecut_e / Ha

        PairDensity.__init__(self, calc, ecut, world=world, nblocks=nblocks,
                             gate_voltage=gate_voltage, txt=txt)

        self.gate_voltage = gate_voltage
        ecut /= Ha

        self.xc = xc
        self.density_cut = density_cut
        self.av_scheme = av_scheme
        self.fxc_mode = fxc_mode
        self.do_GW_too = do_GW_too

        if self.av_scheme is not None:
            assert self.av_scheme == 'wavevector'

        if not self.fxc_mode == 'GW':
            assert self.xc != 'RPA'

        if self.do_GW_too:
            assert self.xc != 'RPA'
            assert self.fxc_mode != 'GW'
            if restartfile is not None:
                raise RuntimeError('Restart function does not currently work '
                                   'with do_GW_too=True.')

        if Eg is None and self.xc == 'JGMsx':
            from ase.dft.bandgap import get_band_gap
            gap, k1, k2 = get_band_gap(self.calc)
            self.Eg = gap
        else:
            self.Eg = Eg

        set_flags(self)

        self.filename = filename
        self.restartfile = restartfile
        self.savew = savew
        self.savepckl = savepckl
        self.ppa = ppa
        self.truncation = truncation
        self.integrate_gamma = integrate_gamma
        self.eta = eta / Ha
        self.E0 = E0 / Ha
        self.domega0 = domega0 / Ha
        self.omega2 = omega2 / Ha
        if anisotropy_correction is not None:
            self.ac = anisotropy_correction
            warnings.warn('anisotropy_correction changed name to '
                          'q0_correction. Please update your script(s).')
        else:
            self.ac = q0_correction

        if self.ac:
            assert self.truncation == '2D'
            self.x0density = 0.1  # ? 0.01

        self.maxiter = maxiter
        self.method = method
        self.mixing = mixing
        if self.method == 'GW0':
            assert self.maxiter > 1

        self.kpts = list(select_kpts(kpts, self.calc))

        if bands is not None and relbands is not None:
            raise ValueError('Use bands or relbands!')

        if relbands is not None:
            bands = [self.calc.wfs.nvalence // 2 + b for b in relbands]

        if bands is None:
            bands = [0, self.nocc2]

        self.bands = bands

        self.ibzk_kc = self.calc.get_ibz_k_points()
        nibzk = len(self.ibzk_kc)
        self.eps0_skn = np.array([[self.calc.get_eigenvalues(kpt=k, spin=s)
                                   for k in range(nibzk)]
                                  for s in range(self.calc.wfs.nspins)]) / Ha

        b1, b2 = bands
        self.shape = shape = (self.calc.wfs.nspins, len(self.kpts), b2 - b1)
        self.eps_skn = np.empty(shape)     # KS-eigenvalues
        self.f_skn = np.empty(shape)       # occupation numbers
        self.sigma_skn = np.zeros(shape)   # self-energies
        self.dsigma_skn = np.zeros(shape)  # derivatives of self-energies
        self.vxc_skn = None                # KS XC-contributions
        self.exx_skn = None                # exact exchange contributions
        self.Z_skn = None                  # renormalization factors

        if nbands is None:
            nbands = int(self.vol * ecut**1.5 * 2**0.5 / 3 / pi**2)
        self.nbands = nbands
        self.nspins = self.calc.wfs.nspins

        if self.nspins != 1 and self.fxc_mode != 'GW':
            raise RuntimeError('Including a xc kernel does currently not '
                               'work for spinpolarized systems.')

        p()
        p('Quasi particle states:')
        if kpts is None:
            p('All k-points in IBZ')
        else:
            kptstxt = ', '.join(['{0:d}'.format(k) for k in self.kpts])
            p('k-points (IBZ indices): [' + kptstxt + ']')
        p('Band range: ({0:d}, {1:d})'.format(b1, b2))
        p()
        p('Computational parameters:')
        if not ecut_extrapolation:
            p('Plane wave cut-off: {0:g} eV'.format(self.ecut * Ha))
        else:
            p('Extrapolating to infinite plane wave cut-off using points at:')
            p('  [%.3f, %.3f, %.3f] eV' % tuple(self.ecut_e[:] * Ha))
        p('Number of bands: {0:d}'.format(self.nbands))
        p('Coulomb cutoff:', self.truncation)
        p('Broadening: {0:g} eV'.format(self.eta * Ha))
        p()
        p('Self-consistency method:', self.method)
        p('fxc mode:', self.fxc_mode)
        p('Kernel:', self.xc)
        p('Averaging scheme:', self.av_scheme)
        p('Do GW too:', self.do_GW_too)

        if self.method == 'GW0':
            p('Number of iterations:', self.maxiter)
            p('Mixing:', self.mixing)
        p()
        kd = self.calc.wfs.kd

        self.mysKn1n2 = None  # my (s, K, n1, n2) indices
        self.distribute_k_points_and_bands(b1, b2, kd.ibz2bz_k[self.kpts])

        # Find q-vectors and weights in the IBZ:
        assert -1 not in kd.bz2bz_ks
        offset_c = 0.5 * ((kd.N_c + 1) % 2) / kd.N_c
        bzq_qc = monkhorst_pack(kd.N_c) + offset_c
        self.qd = KPointDescriptor(bzq_qc)
        self.qd.set_symmetry(self.calc.atoms, kd.symmetry)

        txt.flush()

    @timer('G0W0')
    def calculate(self):
        """Starts the G0W0 calculation.

        Returns a dict with the results with the following key/value pairs:

        ===========  =============================================
        key          value
        ===========  =============================================
        ``f``        Occupation numbers
        ``eps``      Kohn-Sham eigenvalues in eV
        ``vxc``      Exchange-correlation
                     contributions in eV
        ``exx``      Exact exchange contributions in eV
        ``sigma``    Self-energy contributions in eV
        ``dsigma``   Self-energy derivatives
        ``sigma_e``  Self-energy contributions in eV
                     used for ecut extrapolation
        ``Z``        Renormalization factors
        ``qp``       Quasi particle (QP) energies in eV
        ``iqp``      GW0/GW: QP energies for each iteration in eV
        ===========  =============================================

        All the values are ``ndarray``'s of shape
        (spins, IBZ k-points, bands)."""

        kd = self.calc.wfs.kd

        self.calculate_ks_xc_contribution()
        self.calculate_exact_exchange()

        if self.restartfile is not None:
            loaded = self.load_restart_file()
            if not loaded:
                self.last_q = -1
                self.previous_sigma = 0.
                self.previous_dsigma = 0.

            else:
                print('Reading ' + str(self.last_q + 1) +
                      ' q-point(s) from the previous calculation: ' +
                      self.restartfile + '.sigma.pckl', file=self.fd)
        else:
            self.last_q = -1
            self.previous_sigma = 0.
            self.previous_dsigma = 0.

        self.fd.flush()

        self.ite = 0

        while self.ite < self.maxiter:
            # Reset calculation
            # self-energies
            self.sigma_eskn = np.zeros((len(self.ecut_e), ) + self.shape)
            # derivatives of self-energies
            self.dsigma_eskn = np.zeros((len(self.ecut_e), ) + self.shape)

            if self.do_GW_too:
                self.sigma_GW_eskn = np.zeros((len(self.ecut_e), ) +
                                              self.shape)
                self.dsigma_GW_eskn = np.zeros((len(self.ecut_e), ) +
                                               self.shape)

            # Get KS eigenvalues and occupation numbers:
            if self.ite == 0:
                b1, b2 = self.bands
                nibzk = self.calc.wfs.kd.nibzkpts
                for i, k in enumerate(self.kpts):
                    for s in range(self.nspins):
                        u = s * nibzk + k
                        kpt = self.calc.wfs.kpt_u[u]
                        self.eps_skn[s, i] = kpt.eps_n[b1:b2]
                        self.f_skn[s, i] = kpt.f_n[b1:b2] / kpt.weight

                self.qp_skn = self.eps_skn.copy()
                self.qp_iskn = np.array([self.qp_skn])
                if self.do_GW_too:
                    self.qp_GW_skn = self.eps_skn.copy()
                    self.qp_GW_iskn = np.array([self.qp_GW_skn])

            if self.ite > 0:
                self.update_energies(mixing=self.mixing)

            # My part of the states we want to calculate QP-energies for:
            mykpts = [self.get_k_point(s, K, n1, n2)
                      for s, K, n1, n2 in self.mysKn1n2]
            nkpt = len(mykpts)

            # Loop over q in the IBZ:
            nQ = 0
            for ie, pd0, W0, q_c, m2, W0_GW in \
                    self.calculate_screened_potential():
                if nQ == 0:
                    print('Summing all q:', file=self.fd)
                    pb = ProgressBar(self.fd)
                for u, kpt1 in enumerate(mykpts):
                    pb.update((nQ + 1) * u / nkpt / len(self.qd))
                    K2 = kd.find_k_plus_q(q_c, [kpt1.K])[0]
                    kpt2 = self.get_k_point(kpt1.s, K2, 0, m2,
                                            block=True)
                    k1 = kd.bz2ibz_k[kpt1.K]
                    i = self.kpts.index(k1)

                    self.calculate_q(ie, i, kpt1, kpt2, pd0, W0, W0_GW)
                nQ += 1
            pb.finish()

            self.world.sum(self.sigma_eskn)
            self.world.sum(self.dsigma_eskn)

            if self.do_GW_too:
                self.world.sum(self.sigma_GW_eskn)
                self.world.sum(self.dsigma_GW_eskn)

            if self.restartfile is not None and loaded:
                self.sigma_eskn += self.previous_sigma
                self.dsigma_eskn += self.previous_dsigma

            if len(self.ecut_e) > 1:  # interpolate to infinite ecut
                self.extrapolate_ecut()
            else:
                self.sigma_skn = self.sigma_eskn[0]
                self.dsigma_skn = self.dsigma_eskn[0]
                if self.do_GW_too:
                    self.sigma_GW_skn = self.sigma_GW_eskn[0]
                    self.dsigma_GW_skn = self.dsigma_GW_eskn[0]

            self.Z_skn = 1 / (1 - self.dsigma_skn)

            qp_skn = self.qp_skn + self.Z_skn * (
                self.eps_skn -
                self.vxc_skn - self.qp_skn + self.exx_skn +
                self.sigma_skn)

            self.qp_skn = qp_skn

            self.qp_iskn = np.concatenate((self.qp_iskn,
                                           np.array([self.qp_skn])))

            if self.do_GW_too:
                self.Z_GW_skn = 1 / (1 - self.dsigma_GW_skn)

                qp_GW_skn = self.qp_GW_skn + self.Z_GW_skn * (
                    self.eps_skn -
                    self.vxc_skn - self.qp_GW_skn + self.exx_skn +
                    self.sigma_GW_skn)

                self.qp_GW_skn = qp_GW_skn

            self.ite += 1

        results = {'f': self.f_skn,
                   'eps': self.eps_skn * Ha,
                   'vxc': self.vxc_skn * Ha,
                   'exx': self.exx_skn * Ha,
                   'sigma': self.sigma_skn * Ha,
                   'dsigma': self.dsigma_skn,
                   'Z': self.Z_skn,
                   'qp': self.qp_skn * Ha,
                   'iqp': self.qp_iskn * Ha}

        if self.do_GW_too:
            self.results_GW = {'f': self.f_skn,
                               'eps': self.eps_skn * Ha,
                               'vxc': self.vxc_skn * Ha,
                               'exx': self.exx_skn * Ha,
                               'sigma': self.sigma_GW_skn * Ha,
                               'dsigma': self.dsigma_GW_skn,
                               'Z': self.Z_GW_skn,
                               'qp': self.qp_GW_skn * Ha,
                               'iqp': self.qp_GW_iskn * Ha}

        self.print_results(results)

        if len(self.ecut_e) > 1:
            # save non-extrapolated result and R^2 value for fit quality.
            results.update({'sigma_eskn': self.sigma_eskn * Ha,
                            'dsigma_eskn': self.dsigma_eskn * Ha,
                            'sigr2_skn': self.sigr2_skn,
                            'dsigr2_skn': self.dsigr2_skn})

            if self.do_GW_too:
                self.results_GW.update(
                    {'sigma_GW_eskn': self.sigma_GW_eskn * Ha,
                     'dsigma_GW_eskn': self.dsigma_GW_eskn * Ha,
                     'sigr2_GW_skn': self.sigr2_GW_skn,
                     'dsigr2_GW_skn': self.dsigr2_GW_skn})

        if self.savepckl:
            with paropen(self.filename + '_results.pckl', 'wb') as fd:
                pickle.dump(results, fd, 2)
            if self.do_GW_too:
                with paropen(self.filename + '_results_GW.pckl', 'wb') as fd:
                    pickle.dump(self.results_GW, fd, 2)

        # After we have written the results restartfile is obsolete
        if self.restartfile is not None:
            if self.world.rank == 0:
                if os.path.isfile(self.restartfile + '.sigma.pckl'):
                    os.remove(self.restartfile + '.sigma.pckl')

        if self.method == 'GW0' and self.world.rank == 0:
            for iq in range(len(self.qd.ibzk_kc)):
                try:
                    os.remove(self.filename + '.rank' +
                              str(self.blockcomm.rank) + '.w.q' +
                              str(iq) + '.pckl')
                except OSError:
                    pass

        return results

    def calculate_q(self, ie, k, kpt1, kpt2, pd0, W0, W0_GW=None):
        """Calculates the contribution to the self-energy and its derivative
        for a given set of k-points, kpt1 and kpt2."""

        if W0_GW is None:
            Ws = [W0]
        else:
            Ws = [W0, W0_GW]

        wfs = self.calc.wfs

        N_c = pd0.gd.N_c
        i_cG = self.sign * np.dot(self.U_cc,
                                  np.unravel_index(pd0.Q_qG[0], N_c))

        q_c = wfs.kd.bzk_kc[kpt2.K] - wfs.kd.bzk_kc[kpt1.K]

        shift0_c = q_c - self.sign * np.dot(self.U_cc, pd0.kd.bzk_kc[0])
        assert np.allclose(shift0_c.round(), shift0_c)
        shift0_c = shift0_c.round().astype(int)

        shift_c = kpt1.shift_c - kpt2.shift_c - shift0_c
        I_G = np.ravel_multi_index(i_cG + shift_c[:, None], N_c, 'wrap')

        G_Gv = pd0.get_reciprocal_vectors()
        pos_av = np.dot(self.spos_ac, pd0.gd.cell_cv)
        M_vv = np.dot(pd0.gd.cell_cv.T,
                      np.dot(self.U_cc.T,
                             np.linalg.inv(pd0.gd.cell_cv).T))

        Q_aGii = []
        for a, Q_Gii in enumerate(self.Q_aGii):
            x_G = np.exp(1j * np.dot(G_Gv, (pos_av[a] -
                                            np.dot(M_vv, pos_av[a]))))
            U_ii = self.calc.wfs.setups[a].R_sii[self.s]
            Q_Gii = np.dot(np.dot(U_ii, Q_Gii * x_G[:, None, None]),
                           U_ii.T).transpose(1, 0, 2)
            if self.sign == -1:
                Q_Gii = Q_Gii.conj()
            Q_aGii.append(Q_Gii)

        if debug:
            self.check(ie, i_cG, shift0_c, N_c, q_c, Q_aGii)

        if self.ppa:
            calculate_sigma = self.calculate_sigma_ppa
        else:
            calculate_sigma = self.calculate_sigma

        for n in range(kpt1.n2 - kpt1.n1):
            ut1cc_R = kpt1.ut_nR[n].conj()
            eps1 = kpt1.eps_n[n]
            C1_aGi = [np.dot(Qa_Gii, P1_ni[n].conj())
                      for Qa_Gii, P1_ni in zip(Q_aGii, kpt1.P_ani)]
            n_mG = self.calculate_pair_densities(ut1cc_R, C1_aGi, kpt2,
                                                 pd0, I_G)
            if self.sign == 1:
                n_mG = n_mG.conj()

            f_m = kpt2.f_n
            deps_m = eps1 - kpt2.eps_n

            nn = kpt1.n1 + n - self.bands[0]

            for jj, W in enumerate(Ws):
                sigma, dsigma = calculate_sigma(n_mG, deps_m, f_m, W)
                if jj == 0:
                    self.sigma_eskn[ie, kpt1.s, k, nn] += sigma
                    self.dsigma_eskn[ie, kpt1.s, k, nn] += dsigma
                else:
                    self.sigma_GW_eskn[ie, kpt1.s, k, nn] += sigma
                    self.dsigma_GW_eskn[ie, kpt1.s, k, nn] += dsigma

    def check(self, ie, i_cG, shift0_c, N_c, q_c, Q_aGii):
        I0_G = np.ravel_multi_index(i_cG - shift0_c[:, None], N_c, 'wrap')
        qd1 = KPointDescriptor([q_c])
        pd1 = PWDescriptor(self.ecut_e[ie], self.calc.wfs.gd, complex, qd1)
        G_I = np.empty(N_c.prod(), int)
        G_I[:] = -1
        I1_G = pd1.Q_qG[0]
        G_I[I1_G] = np.arange(len(I0_G))
        G_G = G_I[I0_G]
        assert len(I0_G) == len(I1_G)
        assert (G_G >= 0).all()
        for a, Q_Gii in enumerate(self.initialize_paw_corrections(pd1)):
            e = abs(Q_aGii[a] - Q_Gii[G_G]).max()
            assert e < 1e-12

    @timer('Sigma')
    def calculate_sigma(self, n_mG, deps_m, f_m, C_swGG):
        """Calculates a contribution to the self-energy and its derivative for
        a given (k, k-q)-pair from its corresponding pair-density and
        energy."""
        o_m = abs(deps_m)
        # Add small number to avoid zeros for degenerate states:
        sgn_m = np.sign(deps_m + 1e-15)

        # Pick +i*eta or -i*eta:
        s_m = (1 + sgn_m * np.sign(0.5 - f_m)).astype(int) // 2

        beta = (2**0.5 - 1) * self.domega0 / self.omega2
        w_m = (o_m / (self.domega0 + beta * o_m)).astype(int)
        m_inb = np.where(w_m < len(self.omega_w) - 1)[0]
        o1_m = np.empty(len(o_m))
        o2_m = np.empty(len(o_m))
        o1_m[m_inb] = self.omega_w[w_m[m_inb]]
        o2_m[m_inb] = self.omega_w[w_m[m_inb] + 1]

        x = 1.0 / (self.qd.nbzkpts * 2 * pi * self.vol)
        sigma = 0.0
        dsigma = 0.0
        # Performing frequency integration
        for o, o1, o2, sgn, s, w, n_G in zip(o_m, o1_m, o2_m,
                                             sgn_m, s_m, w_m, n_mG):

            if w >= len(self.omega_w) - 1:
                continue

            C1_GG = C_swGG[s][w]
            C2_GG = C_swGG[s][w + 1]
            p = x * sgn
            myn_G = n_G[self.Ga:self.Gb]

            sigma1 = p * np.dot(np.dot(myn_G, C1_GG), n_G.conj()).imag
            sigma2 = p * np.dot(np.dot(myn_G, C2_GG), n_G.conj()).imag
            sigma += ((o - o1) * sigma2 + (o2 - o) * sigma1) / (o2 - o1)
            dsigma += sgn * (sigma2 - sigma1) / (o2 - o1)

        return sigma, dsigma

    def calculate_screened_potential(self):
        """Calculates the screened potential for each q-point in the 1st BZ.
        Since many q-points are related by symmetry, the actual calculation is
        only done for q-points in the IBZ and the rest are obtained by symmetry
        transformations. Results are returned as a generator to that it is not
        necessary to store a huge matrix for each q-point in the memory."""
        # The decorator $timer('W') doesn't work for generators, do we will
        # have to manually start and stop the timer here:
        self.timer.start('W')
        if self.ite == 0:
            print('\nCalculating screened Coulomb potential', file=self.fd)
            if self.truncation is not None:
                print('Using %s truncated Coloumb potential' % self.truncation,
                      file=self.fd)

        if self.ppa:
            if self.ite == 0:
                print('Using Godby-Needs plasmon-pole approximation:',
                      file=self.fd)
                print('  Fitting energy: i*E0, E0 = %.3f Hartee' % self.E0,
                      file=self.fd)

            # use small imaginary frequency to avoid dividing by zero:
            frequencies = [1e-10j, 1j * self.E0 * Ha]

            parameters = {'eta': 0,
                          'hilbert': False,
                          'timeordered': False,
                          'frequencies': frequencies}
        else:
            if self.ite == 0:
                print('Using full frequency integration:', file=self.fd)
                print('  domega0: {0:g}'.format(self.domega0 * Ha),
                      file=self.fd)
                print('  omega2: {0:g}'.format(self.omega2 * Ha),
                      file=self.fd)

            parameters = {'eta': self.eta * Ha,
                          'hilbert': True,
                          'timeordered': True,
                          'domega0': self.domega0 * Ha,
                          'omega2': self.omega2 * Ha}

        self.fd.flush()

        chi0 = Chi0(self.inputcalc,
                    nbands=self.nbands,
                    ecut=self.ecut * Ha,
                    intraband=False,
                    real_space_derivatives=False,
                    txt=self.filename + '.w.txt',
                    timer=self.timer,
                    nblocks=self.blockcomm.size,
                    gate_voltage=self.gate_voltage,
                    **parameters)

        if self.truncation == 'wigner-seitz':
            wstc = WignerSeitzTruncatedCoulomb(
                self.calc.wfs.gd.cell_cv,
                self.calc.wfs.kd.N_c,
                chi0.fd)
        else:
            wstc = None

        self.omega_w = chi0.omega_w
        self.omegamax = chi0.omegamax

        htp = HilbertTransform(self.omega_w, self.eta, gw=True)
        htm = HilbertTransform(self.omega_w, -self.eta, gw=True)

        # Find maximum size of chi-0 matrices:
        gd = self.calc.wfs.gd
        nGmax = max(count_reciprocal_vectors(self.ecut, gd, q_c)
                    for q_c in self.qd.ibzk_kc)
        nw = len(self.omega_w)

        size = self.blockcomm.size

        mynGmax = (nGmax + size - 1) // size
        mynw = (nw + size - 1) // size

        # some memory sizes...
        if self.world.rank == 0:
            # A1_x, A2_x
            siz = (nw * mynGmax * nGmax +
                   max(mynw * nGmax, nw * mynGmax) * nGmax) * 16
            sizA = (nw * nGmax * nGmax + nw * nGmax * nGmax) * 16
            print('  memory estimate for chi0: local=%.2f MB, global=%.2f MB'
                  % (siz / 1024**2, sizA / 1024**2), file=self.fd)
            self.fd.flush()

        # Allocate memory in the beginning and use for all q:
        A1_x = np.empty(nw * mynGmax * nGmax, complex)
        A2_x = np.empty(max(mynw * nGmax, nw * mynGmax) * nGmax, complex)

        # Need to pause the timer in between iterations
        self.timer.stop('W')
        for iq, q_c in enumerate(self.qd.ibzk_kc):
            if iq <= self.last_q:
                continue

            thisqd = KPointDescriptor([q_c])
            pd = PWDescriptor(self.ecut, self.calc.wfs.gd, complex, thisqd)
            nG = pd.ngmax
            mynG = (nG + self.blockcomm.size - 1) // self.blockcomm.size
            chi0.Ga = self.blockcomm.rank * mynG
            chi0.Gb = min(chi0.Ga + mynG, nG)
            if len(self.ecut_e) > 1:
                shape = (nw, chi0.Gb - chi0.Ga, nG)
                chi0bands_wGG = A1_x[:np.prod(shape)].reshape(shape).copy()
                chi0bands_wGG[:] = 0.0

                if np.allclose(q_c, 0.0):
                    chi0bands_wxvG = np.zeros((nw, 2, 3, nG), complex)
                    chi0bands_wvv = np.zeros((nw, 3, 3), complex)
                else:
                    chi0bands_wxvG = None
                    chi0bands_wvv = None
            else:
                chi0bands_wGG = None
                chi0bands_wxvG = None
                chi0bands_wvv = None

            m1 = chi0.nocc1
            for ie, ecut in enumerate(self.ecut_e):
                self.timer.start('W')
                if self.savew:
                    wfilename = (self.filename + '.rank' +
                                 str(self.blockcomm.rank) +
                                 '.w.q%d.pckl' % iq)
                    fd = opencew(wfilename)
                if self.savew and fd is None:
                    # Read screened potential from file
                    with open(wfilename, 'rb') as fd:
                        pdi, W = pickleload(fd)
                    # We also need to initialize the PAW corrections
                    self.Q_aGii = self.initialize_paw_corrections(pdi)

                    nG = pdi.ngmax
                    nw = len(self.omega_w)
                    mynG = (nG + self.blockcomm.size - 1) // \
                        self.blockcomm.size
                    self.Ga = self.blockcomm.rank * mynG
                    self.Gb = min(self.Ga + mynG, nG)
                    assert mynG * (self.blockcomm.size - 1) < nG
                else:
                    # First time calculation
                    if ecut == self.ecut:
                        # Nothing to cut away:
                        m2 = self.nbands
                    else:
                        m2 = int(self.vol * ecut**1.5 * 2**0.5 / 3 / pi**2)

                    pdi, W, W_GW = self.calculate_w(
                        chi0, q_c, pd, chi0bands_wGG,
                        chi0bands_wxvG, chi0bands_wvv,
                        m1, m2, ecut, htp, htm, wstc,
                        A1_x, A2_x, iq)
                    m1 = m2
                    if self.savew:
                        if self.blockcomm.size > 1:
                            thisfile = (self.filename + '.rank' +
                                        str(self.blockcomm.rank) +
                                        '.w.q%d.pckl' % iq)
                            with open(thisfile, 'wb') as fd:
                                pickle.dump((pdi, W), fd, 2)
                        else:
                            pickle.dump((pdi, W), fd, 2)

                self.timer.stop('W')
                # Loop over all k-points in the BZ and find those that are
                # related to the current IBZ k-point by symmetry
                Q1 = self.qd.ibz2bz_k[iq]
                done = set()
                for Q2 in self.qd.bz2bz_ks[Q1]:
                    if Q2 >= 0 and Q2 not in done:
                        s = self.qd.sym_k[Q2]
                        self.s = s
                        self.U_cc = self.qd.symmetry.op_scc[s]
                        time_reversal = self.qd.time_reversal_k[Q2]
                        self.sign = 1 - 2 * time_reversal
                        Q_c = self.qd.bzk_kc[Q2]
                        d_c = self.sign * np.dot(self.U_cc, q_c) - Q_c
                        assert np.allclose(d_c.round(), d_c)
                        yield ie, pdi, W, Q_c, m2, W_GW
                        done.add(Q2)

                if self.restartfile is not None:
                    self.save_restart_file(iq)

    @timer('WW')
    def calculate_w(self, chi0, q_c, pd, chi0bands_wGG, chi0bands_wxvG,
                    chi0bands_wvv, m1, m2, ecut, htp, htm, wstc, A1_x, A2_x,
                    iq):
        """Calculates the screened potential for a specified q-point."""

        nw = len(self.omega_w)
        nG = pd.ngmax
        mynG = (nG + self.blockcomm.size - 1) // self.blockcomm.size
        self.Ga = chi0.Ga
        self.Gb = chi0.Gb
        shape = (nw, chi0.Gb - chi0.Ga, nG)
        # construct empty matrix for chi
        chi0_wGG = A1_x[:np.prod(shape)].reshape(shape).copy()
        chi0_wGG[:] = 0.0
        if np.allclose(q_c, 0.0):
            chi0_wxvG = np.zeros((nw, 2, 3, nG), complex)
            chi0_wvv = np.zeros((nw, 3, 3), complex)
        else:
            chi0_wxvG = None
            chi0_wvv = None

        chi0._calculate(pd, chi0_wGG, chi0_wxvG, chi0_wvv, m1, m2,
                        range(self.nspins), extend_head=False)

        if len(self.ecut_e) > 1:
            # Add chi from previous cutoff with remaining bands
            chi0_wGG += chi0bands_wGG
            chi0bands_wGG[:] = chi0_wGG.copy()
            if np.allclose(q_c, 0.0):
                chi0_wxvG += chi0bands_wxvG
                chi0bands_wxvG[:] = chi0_wxvG.copy()
                chi0_wvv += chi0bands_wvv
                chi0bands_wvv[:] = chi0_wvv.copy()

        self.Q_aGii = chi0.Q_aGii

        mynw = (nw + self.blockcomm.size - 1) // self.blockcomm.size
        if self.blockcomm.size > 1:
            chi0_wGG = chi0.redistribute(chi0_wGG, A2_x)
            wa = min(self.blockcomm.rank * mynw, nw)
            wb = min(wa + mynw, nw)
        else:
            wa = 0
            wb = nw

        if ecut == pd.ecut:
            nG = len(chi0_wGG[0])
            pdi = pd
            G2G = None

        elif ecut < pd.ecut:  # construct subset chi0 matrix with lower ecut
            pdi = PWDescriptor(ecut, pd.gd, dtype=pd.dtype,
                               kd=pd.kd)
            nG = pdi.ngmax
            mynG = (nG + self.blockcomm.size - 1) // self.blockcomm.size
            self.Ga = self.blockcomm.rank * mynG
            self.Gb = min(self.Ga + mynG, nG)
            nw = len(self.omega_w)
            mynw = (nw + self.blockcomm.size - 1) // self.blockcomm.size

            G2G = PWMapping(pdi, pd).G2_G1
            chi0_wGG = chi0_wGG.take(G2G, axis=1).take(G2G, axis=2)

            if chi0_wxvG is not None:
                chi0_wxvG = chi0_wxvG.take(G2G, axis=3)

            if self.Q_aGii is not None:
                for a, Q_Gii in enumerate(self.Q_aGii):
                    self.Q_aGii[a] = Q_Gii.take(G2G, axis=0)

        if self.integrate_gamma != 0:
            if self.integrate_gamma == 2:
                reduced = True
            else:
                reduced = False
            V0, sqrV0 = get_integrated_kernel(pdi,
                                              self.calc.wfs.kd.N_c,
                                              truncation=self.truncation,
                                              reduced=reduced,
                                              N=100)
        elif self.integrate_gamma == 0 and np.allclose(q_c, 0):
            bzvol = (2 * np.pi)**3 / self.vol / self.qd.nbzkpts
            Rq0 = (3 * bzvol / (4 * np.pi))**(1. / 3.)
            V0 = 16 * np.pi**2 * Rq0 / bzvol
            sqrV0 = (4 * np.pi)**(1.5) * Rq0**2 / bzvol / 2
        else:
            pass

        delta_GG = np.eye(nG)

        if self.ppa:
            einv_wGG = []

        # Calculate kernel
        fv = calculate_kernel(self, nG, self.nspins, iq, G2G)[0:nG, 0:nG]
        # Generate fine grid in vicinity of gamma
        if np.allclose(q_c, 0):
            kd = self.calc.wfs.kd
            N = 4
            N_c = np.array([N, N, N])
            if self.truncation is not None:
                # Only average periodic directions if trunction is used
                N_c[np.where(kd.N_c == 1)[0]] = 1
            qf_qc = monkhorst_pack(N_c) / kd.N_c
            qf_qc *= 1.0e-6
            U_scc = kd.symmetry.op_scc
            qf_qc = kd.get_ibz_q_points(qf_qc, U_scc)[0]
            weight_q = kd.q_weights
            qf_qv = 2 * np.pi * np.dot(qf_qc, pd.gd.icell_cv)
            a_wq = np.sum([chi0_vq * qf_qv.T
                           for chi0_vq in
                           np.dot(chi0_wvv[wa:wb], qf_qv.T)], axis=1)
            a0_qwG = np.dot(qf_qv, chi0_wxvG[wa:wb, 0])
            a1_qwG = np.dot(qf_qv, chi0_wxvG[wa:wb, 1])

        self.timer.start('Dyson eq.')
        # Calculate W and store it in chi0_wGG ndarray:
        if self.do_GW_too:
            chi0_GW_wGG = chi0_wGG.copy()
        else:
            chi0_GW_wGG = [0]

        for iw, [chi0_GG, chi0_GW_GG] in enumerate(
            zip(chi0_wGG,
                itertools.cycle(chi0_GW_wGG))):
            if np.allclose(q_c, 0):
                einv_GG = np.zeros((nG, nG), complex)
                if self.do_GW_too:
                    einv_GW_GG = np.zeros((nG, nG), complex)
                for iqf in range(len(qf_qv)):
                    chi0_GG[0] = a0_qwG[iqf, iw]
                    chi0_GG[:, 0] = a1_qwG[iqf, iw]
                    chi0_GG[0, 0] = a_wq[iw, iqf]
                    if self.do_GW_too:
                        chi0_GW_GG[0] = a0_qwG[iqf, iw]
                        chi0_GW_GG[:, 0] = a1_qwG[iqf, iw]
                        chi0_GW_GG[0, 0] = a_wq[iw, iqf]
                    sqrV_G = get_coulomb_kernel(pdi,
                                                kd.N_c,
                                                truncation=self.truncation,
                                                wstc=wstc,
                                                q_v=qf_qv[iqf])**0.5

#                    chi0v_GG = chi0_GG * sqrV_G * sqrV_G[:, np.newaxis]
#                    if self.nspins == 2:
#                        chi0v = np.zeros((2 * nG, 2 * nG), dtype=complex)
#                        for s in range(self.nspins):
#                            m = s * nG
#                            n = (s + 1) * nG
#                            chi0v[m:n, m:n] = chi0v_GG
#                    else:
#                    chi0v = chi0v_GG

                    if self.fxc_mode == 'GWP':
                        e_GG = (np.eye(nG) -
                                np.dot(
                                    np.linalg.inv(
                                        np.eye(nG) -
                                        np.dot(chi0_GG *
                                               sqrV_G *
                                               sqrV_G[:, np.newaxis], fv) +
                                        chi0_GG * sqrV_G *
                                        sqrV_G[:, np.newaxis]),
                                    chi0_GG * sqrV_G *
                                    sqrV_G[:, np.newaxis]))
                    elif self.fxc_mode == 'GWS':
                        e_GG = np.dot(
                            np.linalg.inv(
                                np.eye(nG) +
                                np.dot(chi0_GG *
                                       sqrV_G *
                                       sqrV_G[:, np.newaxis], fv) -
                                chi0_GG * sqrV_G *
                                sqrV_G[:, np.newaxis]),
                            np.eye(nG) -
                            chi0_GG * sqrV_G *
                            sqrV_G[:, np.newaxis])
                    else:
                        e_GG = np.eye(nG) - np.dot(chi0_GG * sqrV_G *
                                                   sqrV_G[:, np.newaxis], fv)

                    einv_GG += np.linalg.inv(e_GG) * weight_q[iqf]

                    if self.do_GW_too:
                        e_GW_GG = (np.eye(nG) -
                                   chi0_GW_GG * sqrV_G *
                                   sqrV_G[:, np.newaxis])
                        einv_GW_GG += np.linalg.inv(e_GW_GG) * weight_q[iqf]

            else:
                sqrV_G = get_coulomb_kernel(pdi,
                                            self.calc.wfs.kd.N_c,
                                            truncation=self.truncation,
                                            wstc=wstc)**0.5
                if self.fxc_mode == 'GWP':
                    e_GG = (np.eye(nG) -
                            np.dot(
                                np.linalg.inv(
                                    np.eye(nG) -
                                    np.dot(chi0_GG * sqrV_G *
                                           sqrV_G[:, np.newaxis], fv) +
                                    chi0_GG * sqrV_G *
                                    sqrV_G[:, np.newaxis]),
                                chi0_GG * sqrV_G * sqrV_G[:, np.newaxis]))
                elif self.fxc_mode == 'GWS':
                    e_GG = np.dot(
                        np.linalg.inv(
                            np.eye(nG) +
                            np.dot(chi0_GG * sqrV_G *
                                   sqrV_G[:, np.newaxis], fv) -
                            chi0_GG * sqrV_G *
                            sqrV_G[:, np.newaxis]),
                        np.eye(nG) - chi0_GG *
                        sqrV_G * sqrV_G[:, np.newaxis])
                else:
                    e_GG = (delta_GG -
                            np.dot(chi0_GG * sqrV_G *
                                   sqrV_G[:, np.newaxis], fv))

                einv_GG = np.linalg.inv(e_GG)

                if self.do_GW_too:
                    e_GW_GG = (delta_GG -
                               chi0_GW_GG * sqrV_G * sqrV_G[:, np.newaxis])
                    einv_GW_GG = np.linalg.inv(e_GW_GG)

            if self.ppa:
                einv_wGG.append(einv_GG - delta_GG)
            else:
                W_GG = chi0_GG
                W_GG[:] = (einv_GG - delta_GG) * sqrV_G * sqrV_G[:, np.newaxis]

                if self.do_GW_too:
                    W_GW_GG = chi0_GW_GG
                    W_GW_GG[:] = ((einv_GW_GG - delta_GG) *
                                  sqrV_G * sqrV_G[:, np.newaxis])

                if self.ac and np.allclose(q_c, 0):
                    if iw == 0:
                        print_ac = True
                    else:
                        print_ac = False
                    self.add_q0_correction(pdi, W_GG, einv_GG,
                                           chi0_wxvG[wa + iw],
                                           chi0_wvv[wa + iw],
                                           sqrV_G,
                                           print_ac=print_ac)
                    if self.do_GW_too:
                        self.add_q0_correction(pdi, W_GW_GG, einv_GW_GG,
                                               chi0_wxvG[wa + iw],
                                               chi0_wvv[wa + iw],
                                               sqrV_G,
                                               print_ac=print_ac)
                elif np.allclose(q_c, 0) or self.integrate_gamma != 0:
                    W_GG[0, 0] = (einv_GG[0, 0] - 1.0) * V0
                    W_GG[0, 1:] = einv_GG[0, 1:] * sqrV_G[1:] * sqrV0
                    W_GG[1:, 0] = einv_GG[1:, 0] * sqrV0 * sqrV_G[1:]
                    if self.do_GW_too:
                        W_GW_GG[0, 0] = (einv_GW_GG[0, 0] - 1.0) * V0
                        W_GW_GG[0, 1:] = einv_GW_GG[0, 1:] * sqrV_G[1:] * sqrV0
                        W_GW_GG[1:, 0] = einv_GW_GG[1:, 0] * sqrV0 * sqrV_G[1:]
                else:
                    pass

        if self.ppa:
            omegat_GG = self.E0 * np.sqrt(einv_wGG[1] /
                                          (einv_wGG[0] - einv_wGG[1]))
            R_GG = -0.5 * omegat_GG * einv_wGG[0]
            W_GG = pi * R_GG * sqrV_G * sqrV_G[:, np.newaxis]
            if np.allclose(q_c, 0) or self.integrate_gamma != 0:
                W_GG[0, 0] = pi * R_GG[0, 0] * V0
                W_GG[0, 1:] = pi * R_GG[0, 1:] * sqrV_G[1:] * sqrV0
                W_GG[1:, 0] = pi * R_GG[1:, 0] * sqrV0 * sqrV_G[1:]

            self.timer.stop('Dyson eq.')
            return pdi, [W_GG, omegat_GG], None

        if self.do_GW_too:
            A1_GW_x = A1_x.copy()
            A2_GW_x = A2_x.copy()

        if self.blockcomm.size > 1:
            Wm_wGG = chi0.redistribute(chi0_wGG, A1_x)
        else:
            Wm_wGG = chi0_wGG

        Wp_wGG = A2_x[:Wm_wGG.size].reshape(Wm_wGG.shape)
        Wp_wGG[:] = Wm_wGG

        with self.timer('Hilbert transform'):
            htp(Wp_wGG)
            htm(Wm_wGG)
        self.timer.stop('Dyson eq.')

        if self.do_GW_too:
            if self.blockcomm.size > 1:
                Wm_GW_wGG = chi0.redistribute(chi0_GW_wGG, A1_GW_x)
            else:
                Wm_GW_wGG = chi0_GW_wGG

            Wp_GW_wGG = A2_GW_x[:Wm_GW_wGG.size].reshape(Wm_GW_wGG.shape)
            Wp_GW_wGG[:] = Wm_GW_wGG

            htp(Wp_GW_wGG)
            htm(Wm_GW_wGG)
            GW_return = [Wp_GW_wGG, Wm_GW_wGG]
        else:
            GW_return = None

        return pdi, [Wp_wGG, Wm_wGG], GW_return

    @timer('Kohn-Sham XC-contribution')
    def calculate_ks_xc_contribution(self):
        name = self.filename + '.vxc.npy'
        fd, self.vxc_skn = self.read_contribution(name)
        if self.vxc_skn is None:
            print('Calculating Kohn-Sham XC contribution', file=self.fd)
            vxc_skn = vxc(self.calc, self.calc.hamiltonian.xc) / Ha
            n1, n2 = self.bands
            self.vxc_skn = vxc_skn[:, self.kpts, n1:n2]
            np.save(fd, self.vxc_skn)

    @timer('EXX')
    def calculate_exact_exchange(self):
        name = self.filename + '.exx.npy'
        fd, self.exx_skn = self.read_contribution(name)
        if self.exx_skn is None:
            print('Calculating EXX contribution', file=self.fd)
            exx = EXX(self.calc, kpts=self.kpts, bands=self.bands,
                      txt=self.filename + '.exx.txt', timer=self.timer)
            exx.calculate()
            self.exx_skn = exx.get_eigenvalue_contributions() / Ha
            np.save(fd, self.exx_skn)

    def read_contribution(self, filename):
        fd = opencew(filename)  # create, exclusive, write
        if fd is not None:
            # File was not there: nothing to read
            return fd, None

        try:
            with open(filename, 'rb') as fd:
                x_skn = np.load(fd)
        except IOError:
            print('Removing broken file:', filename, file=self.fd)
        else:
            print('Read:', filename, file=self.fd)
            if x_skn.shape == self.shape:
                return None, x_skn
            print('Removing bad file (wrong shape of array):', filename,
                  file=self.fd)

        if self.world.rank == 0:
            os.remove(filename)

        return opencew(filename), None

    def print_results(self, results):
        description = ['f:      Occupation numbers',
                       'eps:     KS-eigenvalues [eV]',
                       'vxc:     KS vxc [eV]',
                       'exx:     Exact exchange [eV]',
                       'sigma:   Self-energies [eV]',
                       'dsigma:  Self-energy derivatives',
                       'Z:       Renormalization factors',
                       'qp:      QP-energies [eV]']

        print('\nResults:', file=self.fd)
        for line in description:
            print(line, file=self.fd)

        b1, b2 = self.bands
        names = [line.split(':', 1)[0] for line in description]
        ibzk_kc = self.calc.wfs.kd.ibzk_kc
        for s in range(self.calc.wfs.nspins):
            for i, ik in enumerate(self.kpts):
                print('\nk-point ' +
                      '{0} ({1}): ({2:.3f}, {3:.3f}, {4:.3f})'.format(
                          i, ik, *ibzk_kc[ik]) + '                ' +
                      self.fxc_mode, file=self.fd)
                print('band' +
                      ''.join('{0:>8}'.format(name) for name in names),
                      file=self.fd)
                for n in range(b2 - b1):
                    print('{0:4}'.format(n + b1) +
                          ''.join('{0:8.3f}'.format(results[name][s, i, n])
                                  for name in names),
                          file=self.fd)
                if self.do_GW_too:
                    print(' ' * 67 + 'GW', file=self.fd)
                    for n in range(b2 - b1):
                        print('{0:4}'.format(n + b1) +
                              ''.join('{0:8.3f}'
                                      .format(self.results_GW[name][s, i, n])
                                      for name in names),
                              file=self.fd)

        self.timer.write(self.fd)

    @timer('PPA-Sigma')
    def calculate_sigma_ppa(self, n_mG, deps_m, f_m, W):
        W_GG, omegat_GG = W

        sigma = 0.0
        dsigma = 0.0

        # init variables (is this necessary?)
        nG = n_mG.shape[1]
        deps_GG = np.empty((nG, nG))
        sign_GG = np.empty((nG, nG))
        x1_GG = np.empty((nG, nG))
        x2_GG = np.empty((nG, nG))
        x3_GG = np.empty((nG, nG))
        x4_GG = np.empty((nG, nG))
        x_GG = np.empty((nG, nG))
        dx_GG = np.empty((nG, nG))
        nW_G = np.empty(nG)
        for m in range(np.shape(n_mG)[0]):
            deps_GG = deps_m[m]
            sign_GG = 2 * f_m[m] - 1
            x1_GG = 1 / (deps_GG + omegat_GG - 1j * self.eta)
            x2_GG = 1 / (deps_GG - omegat_GG + 1j * self.eta)
            x3_GG = 1 / (deps_GG + omegat_GG - 1j * self.eta * sign_GG)
            x4_GG = 1 / (deps_GG - omegat_GG - 1j * self.eta * sign_GG)
            x_GG = W_GG * (sign_GG * (x1_GG - x2_GG) + x3_GG + x4_GG)
            dx_GG = W_GG * (sign_GG * (x1_GG**2 - x2_GG**2) +
                            x3_GG**2 + x4_GG**2)
            nW_G = np.dot(n_mG[m], x_GG)
            sigma += np.vdot(n_mG[m], nW_G).real
            nW_G = np.dot(n_mG[m], dx_GG)
            dsigma -= np.vdot(n_mG[m], nW_G).real

        x = 1 / (self.qd.nbzkpts * 2 * pi * self.vol)
        return x * sigma, x * dsigma

    def save_restart_file(self, nQ):
        sigma_eskn_write = self.sigma_eskn.copy()
        dsigma_eskn_write = self.dsigma_eskn.copy()
        self.world.sum(sigma_eskn_write)
        self.world.sum(dsigma_eskn_write)
        data = {'last_q': nQ,
                'sigma_eskn': sigma_eskn_write + self.previous_sigma,
                'dsigma_eskn': dsigma_eskn_write + self.previous_dsigma,
                'kpts': self.kpts,
                'bands': self.bands,
                'nbands': self.nbands,
                'ecut_e': self.ecut_e,
                'domega0': self.domega0,
                'omega2': self.omega2,
                'integrate_gamma': self.integrate_gamma}

        if self.world.rank == 0:
            with open(self.restartfile + '.sigma.pckl', 'wb') as fd:
                pickle.dump(data, fd, 2)

    def load_restart_file(self):
        try:
            with open(self.restartfile + '.sigma.pckl', 'rb') as fd:
                data = pickleload(fd)
        except IOError:
            return False
        else:
            if (data['kpts'] == self.kpts and
                data['bands'] == self.bands and
                data['nbands'] == self.nbands and
                (data['ecut_e'] == self.ecut_e).all and
                data['domega0'] == self.domega0 and
                data['omega2'] == self.omega2 and
                data['integrate_gamma'] == self.integrate_gamma):
                self.last_q = data['last_q']
                self.previous_sigma = data['sigma_eskn']
                self.previous_dsigma = data['dsigma_eskn']
                return True
            else:
                raise ValueError(
                    'Restart file not compatible with parameters used in '
                    'current calculation. Check kpts, bands, nbands, ecut, '
                    'domega0, omega2, integrate_gamma.')

    def extrapolate_ecut(self):
        # Do linear fit of selfenergy vs. inverse of number of plane waves
        # to extrapolate to infinite number of plane waves
        from scipy.stats import linregress
        print('', file=self.fd)
        print('Extrapolating selfenergy to infinite energy cutoff:',
              file=self.fd)
        print('  Performing linear fit to %d points' % len(self.ecut_e),
              file=self.fd)
        self.sigr2_skn = np.zeros(self.shape)
        self.dsigr2_skn = np.zeros(self.shape)
        self.sigma_skn = np.zeros(self.shape)
        self.dsigma_skn = np.zeros(self.shape)
        invN_i = self.ecut_e**(-3. / 2)
        for m in range(np.product(self.shape)):
            s, k, n = np.unravel_index(m, self.shape)

            slope, intercept, r_value, p_value, std_err = \
                linregress(invN_i, self.sigma_eskn[:, s, k, n])

            self.sigr2_skn[s, k, n] = r_value**2
            self.sigma_skn[s, k, n] = intercept

            slope, intercept, r_value, p_value, std_err = \
                linregress(invN_i, self.dsigma_eskn[:, s, k, n])

            self.dsigr2_skn[s, k, n] = r_value**2
            self.dsigma_skn[s, k, n] = intercept

        if np.any(self.sigr2_skn < 0.9) or np.any(self.dsigr2_skn < 0.9):
            print('  Warning: Bad quality of linear fit for some (n,k). ',
                  file=self.fd)
            print('           Higher cutoff might be necesarry.', file=self.fd)

        print('  Minimum R^2 = %1.4f. (R^2 Should be close to 1)' %
              min(np.min(self.sigr2_skn), np.min(self.dsigr2_skn)),
              file=self.fd)

        if self.do_GW_too:
            self.sigr2_GW_skn = np.zeros(self.shape)
            self.dsigr2_GW_skn = np.zeros(self.shape)
            self.sigma_GW_skn = np.zeros(self.shape)
            self.dsigma_GW_skn = np.zeros(self.shape)
            invN_i = self.ecut_e**(-3. / 2)
            for m in range(np.product(self.shape)):
                s, k, n = np.unravel_index(m, self.shape)

                slope, intercept, r_value, p_value, std_err = \
                    linregress(invN_i, self.sigma_GW_eskn[:, s, k, n])

                self.sigr2_GW_skn[s, k, n] = r_value**2
                self.sigma_GW_skn[s, k, n] = intercept

                slope, intercept, r_value, p_value, std_err = \
                    linregress(invN_i, self.dsigma_GW_eskn[:, s, k, n])

                self.dsigr2_GW_skn[s, k, n] = r_value**2
                self.dsigma_GW_skn[s, k, n] = intercept

            if np.any(self.sigr2_GW_skn < 0.9) or np.any(self.dsigr2_GW_skn <
                                                         0.9):
                print('  GW calculation. Warning: Bad quality of linear fit '
                      'for some (n,k). ',
                      file=self.fd)
                print('           Higher cutoff might be necesarry.',
                      file=self.fd)

            print('  Minimum R^2 = %1.4f. (R^2 Should be close to 1)' %
                  min(np.min(self.sigr2_GW_skn), np.min(self.dsigr2_GW_skn)),
                  file=self.fd)

    def add_q0_correction(self, pd, W_GG, einv_GG, chi0_xvG, chi0_vv,
                          sqrV_G, print_ac=False):
        from ase.dft import monkhorst_pack
        self.cell_cv = self.calc.wfs.gd.cell_cv
        self.qpts_qc = self.calc.wfs.kd.bzk_kc
        self.weight_q = 1.0 * np.ones(len(self.qpts_qc)) / len(self.qpts_qc)
        L = self.cell_cv[2, 2]
        vc_G0 = sqrV_G[1:]**2

        B_GG = einv_GG[1:, 1:]
        u_v0G = vc_G0[np.newaxis, :]**0.5 * chi0_xvG[0, :, 1:]
        u_vG0 = vc_G0[np.newaxis, :]**0.5 * chi0_xvG[1, :, 1:]
        U_vv = -chi0_vv
        a_v0G = -np.dot(u_v0G, B_GG)
        a_vG0 = -np.dot(u_vG0, B_GG.T)
        A_vv = U_vv - np.dot(a_v0G, u_vG0.T)
        S_v0G = a_v0G
        S_vG0 = a_vG0
        L_vv = A_vv

        # Get necessary G vectors.
        G_Gv = pd.get_reciprocal_vectors()[1:]
        G_Gv += np.array([1e-14, 1e-14, 0])
        G2_G = np.sum(G_Gv**2, axis=1)
        Gpar_G = np.sum(G_Gv[:, 0:2]**2, axis=1)**0.5

        # Generate numerical q-point grid
        rcell_cv = 2 * pi * np.linalg.inv(self.cell_cv).T
        N_c = self.qd.N_c

        iq = np.argmin(np.sum(self.qpts_qc**2, axis=1))
        assert np.allclose(self.qpts_qc[iq], 0)
        q0cell_cv = np.array([1, 1, 1])**0.5 * rcell_cv / N_c
        q0vol = abs(np.linalg.det(q0cell_cv))

        x0density = self.x0density
        q0density = 2. / L * x0density
        npts_c = np.ceil(np.sum(q0cell_cv**2, axis=1)**0.5 /
                         q0density).astype(int)
        npts_c[2] = 1
        npts_c += (npts_c + 1) % 2
        if print_ac:
            print('Applying analytical 2D correction to W:',
                  file=self.fd)
            print('    Evaluating Gamma point contribution to W on a ' +
                  '%dx%dx%d grid' % tuple(npts_c), file=self.fd)

        qpts_qc = monkhorst_pack(npts_c)
        qgamma = np.argmin(np.sum(qpts_qc**2, axis=1))

        qpts_qv = np.dot(qpts_qc, q0cell_cv)
        qpts_q = np.sum(qpts_qv**2, axis=1)**0.5
        qpts_q[qgamma] = 1e-14
        qdir_qv = qpts_qv / qpts_q[:, np.newaxis]
        qdir_qvv = qdir_qv[:, :, np.newaxis] * qdir_qv[:, np.newaxis, :]
        nq = len(qpts_qc)
        q0area = q0vol / q0cell_cv[2, 2]
        dq0 = q0area / nq
        dq0rad = (dq0 / pi)**0.5
        R = L / 2.
        x0area = q0area * R**2
        dx0rad = dq0rad * R

        exp_q = 4 * pi * (1 - np.exp(-qpts_q * R))
        dv_G = ((pi * L * G2_G * np.exp(-Gpar_G * R) * np.cos(G_Gv[:, 2] * R) -
                 4 * pi * Gpar_G * (1 - np.exp(-Gpar_G * R) *
                                    np.cos(G_Gv[:, 2] * R))) /
                (G2_G**1.5 * Gpar_G *
                 (4 * pi * (1 - np.exp(-Gpar_G * R) *
                            np.cos(G_Gv[:, 2] * R)))**0.5))

        dv_Gv = dv_G[:, np.newaxis] * G_Gv

        # Add corrections
        W_GG[:, 0] = 0.0
        W_GG[0, :] = 0.0

        A_q = np.sum(qdir_qv * np.dot(qdir_qv, L_vv), axis=1)
        frac_q = 1. / (1 + exp_q * A_q)

        # HEAD:
        w00_q = -(exp_q / qpts_q)**2 * A_q * frac_q
        w00_q[qgamma] = 0.0
        W_GG[0, 0] = w00_q.sum() / nq
        Axy = 0.5 * (L_vv[0, 0] + L_vv[1, 1])  # in-plane average
        a0 = 4 * pi * Axy + 1

        W_GG[0, 0] += -((a0 * dx0rad - np.log(a0 * dx0rad + 1)) /
                        a0**2 / x0area * 2 * np.pi * (2 * pi * L)**2 * Axy)

        # WINGS:
        u_q = -exp_q / qpts_q * frac_q
        W_GG[1:, 0] = 1. / nq * np.dot(
            np.sum(qdir_qv * u_q[:, np.newaxis], axis=0),
            S_vG0 * sqrV_G[np.newaxis, 1:])

        W_GG[0, 1:] = 1. / nq * np.dot(
            np.sum(qdir_qv * u_q[:, np.newaxis], axis=0),
            S_v0G * sqrV_G[np.newaxis, 1:])

        # BODY:
        # Constant corrections:
        W_GG[1:, 1:] += 1. / nq * sqrV_G[1:, None] * sqrV_G[None, 1:] * \
            np.tensordot(S_v0G, np.dot(S_vG0.T,
                                       np.sum(-qdir_qvv *
                                              exp_q[:, None, None] *
                                              frac_q[:, None, None],
                                              axis=0)), axes=(0, 1))
        u_vvv = np.tensordot(u_q[:, None] * qpts_qv, qdir_qvv, axes=(0, 0))
        # Gradient corrections:
        W_GG[1:, 1:] += 1. / nq * np.sum(
            dv_Gv[:, :, None] * np.tensordot(
                S_v0G, np.tensordot(u_vvv, S_vG0 * sqrV_G[None, 1:],
                                    axes=(2, 0)), axes=(0, 1)), axis=1)

    def update_energies(self, mixing):
        """Updates the energies of the calculator with the quasi-particle
        energies."""
        shifts_skn = np.zeros(self.shape)
        na, nb = self.bands
        i1 = len(self.qp_iskn) - 2
        i2 = i1 + 1
        if i1 < 0:
            i1 = 0

        for kpt in self.calc.wfs.kpt_u:
            s = kpt.s
            if kpt.k in self.kpts:
                ik = self.kpts.index(kpt.k)
                eps1_n = self.mixer(self.qp_iskn[i1, s, ik],
                                    self.qp_iskn[i2, s, ik],
                                    mixing)
                kpt.eps_n[na:nb] = eps1_n
                """
                Should we do something smart with the bands outside the
                interval?
                Here we shift the unoccupied bands not included by the average
                change of the top-most band and the occupied by the
                bottom-most band included
                """
                shifts_skn[s, ik] = (eps1_n - self.eps0_skn[s, kpt.k, na:nb])

        for kpt in self.calc.wfs.kpt_u:
            s = kpt.s
            if kpt.k in self.kpts:
                ik = self.kpts.index(kpt.k)
                kpt.eps_n[:na] = (self.eps0_skn[s, kpt.k, :na] +
                                  np.mean(shifts_skn[s, :, 0]))
                kpt.eps_n[nb:] = (self.eps0_skn[s, kpt.k, nb:] +
                                  np.mean(shifts_skn[s, :, -1]))
            else:
                """
                kpt.eps_n[:na] = (self.eps0_skn[s, kpt.k, :na] +
                                  np.mean(shifts_skn[s, :, 0]))
                kpt.eps_n[na:nb] = (self.eps0_skn[s, kpt.k, na:nb] +
                                    np.mean(shifts_skn[s, :], axis=0))
                kpt.eps_n[nb:] = (self.eps0_skn[s, kpt.k, nb:] +
                                  np.mean(shifts_skn[s, :, -1]))
                """
                pass

    def mixer(self, e0_skn, e1_skn, mixing=1.0):
        """Mix energies."""
        return e0_skn + mixing * (e1_skn - e0_skn)



from __future__ import print_function

from ase import Atoms
from ase.parallel import parprint
from ase.utils.timing import Timer

from gpaw import GPAW
from gpaw.test import equal
from gpaw.xc.hybrid import HybridXC

timer = Timer()

loa = Atoms('Be2',
            [(0, 0, 0), (2.45, 0, 0)],
            cell=[5.9, 4.8, 5.0])
loa.center()

txt = None
xc = 'PBE0'
nbands = 4

unocc = True
load = False

# usual calculation
fname = 'Be2.gpw'
if not load:
    xco = HybridXC(xc)
    cocc = GPAW(h=0.3,
                eigensolver='rmm-diis',
                xc=xco,
                nbands=nbands,
                convergence={'eigenstates': 1e-4},
                txt=txt)
    cocc.calculate(loa)
else:
    cocc = GPAW(fname)
    cocc.converge_wave_functions()
fo_n = 1. * cocc.get_occupation_numbers()
eo_n = 1. * cocc.get_eigenvalues()

if unocc:
    # apply Fock opeartor also to unoccupied orbitals
    xcu = HybridXC(xc, unocc=True)
    cunocc = GPAW(h=0.3,
                  eigensolver='rmm-diis',
                  xc=xcu,
                  nbands=nbands,
                  convergence={'eigenstates': 1e-4},
                  txt=txt)
    cunocc.calculate(loa)

    parprint('     HF occ          HF unocc      diff')
    parprint('Energy %10.4f   %10.4f %10.4f' %
             (cocc.get_potential_energy(),
              cunocc.get_potential_energy(),
              cocc.get_potential_energy() - cunocc.get_potential_energy()
              ))
    equal(cocc.get_potential_energy(),
          cunocc.get_potential_energy(), 1.e-4)

    fu_n = cunocc.get_occupation_numbers()
    eu_n = cunocc.get_eigenvalues()

    parprint('Eigenvalues:')
    for eo, fo, eu, fu in zip(eo_n, fo_n, eu_n, fu_n):
        parprint('%8.4f %5.2f   %8.4f %5.2f  %8.4f' %
                 (eo, fo, eu, fu, eu - eo))
        if fo > 0.01:
            equal(eo, eu, 3.5e-4)



import numpy as np
from gpaw.utilities.blas import \
    gemm, axpy, r2k, rk, gemmdot, dotc
from gpaw.utilities.tools import tri2full

a = np.arange(5 * 7).reshape(5, 7) + 4.
a2 = np.arange(3 * 7).reshape(3, 7) + 3.
b = np.arange(7) - 2.

# Check gemmdot with floats
assert np.all(np.dot(a, b) == gemmdot(a, b))
assert np.all(np.dot(a, a2.T) == gemmdot(a, a2, trans='t'))
assert np.all(np.dot(a, a2.T) == gemmdot(a, a2, trans='c'))
assert np.dot(b, b) == gemmdot(b, b)

# Check gemmdot with complex arrays
a = a * (2 + 1.j)
a2 = a2 * (-1 + 3.j)
b = b * (3 - 2.j)
assert np.all(np.dot(a, b) == gemmdot(a, b))
assert np.all(np.dot(a, a2.T) == gemmdot(a, a2, trans='t'))
assert np.all(np.dot(a, a2.T.conj()) == gemmdot(a, a2, trans='c'))
assert np.dot(b, b) == gemmdot(b, b, trans='n')
assert np.dot(b, b.conj()) == gemmdot(b, b, trans='c')
assert np.vdot(a, 5.j * a) == dotc(a, 5.j * a)

# Check gemm for transa='n'
a2 = np.arange(7 * 5 * 1 * 3).reshape(7, 5, 1, 3) * (-1. + 4.j) + 3.
c = np.tensordot(a, a2, [1, 0])
gemm(1., a2, a, -1., c, 'n')
assert not c.any()

# Check gemm for transa='c'
a = np.arange(4 * 5 * 1 * 3).reshape(4, 5, 1, 3) * (3. - 2.j) + 4.
c = np.tensordot(a, a2.conj(), [[1, 2, 3], [1, 2, 3]])
gemm(1., a2, a, -1., c, 'c')
assert not c.any()

# Check axpy
c = 5.j * a
axpy(-5.j, a, c)
assert not c.any()

# Check rk
c = np.tensordot(a, a.conj(), [[1, 2, 3], [1, 2, 3]])
rk(1., a, -1., c)
tri2full(c)
assert not c.any()

# Check gemmdot for transa='c'
c = np.tensordot(a, a2.conj(), [-1, -1])
gemmdot(a, a2, beta=-1., out=c, trans='c')
assert not c.any()

# Check gemmdot for transa='n'
a2.shape = 3, 7, 5, 1
c = np.tensordot(a, a2, [-1, 0])
gemmdot(a, a2, beta=-1., out=c, trans='n')
assert not c.any()

# Check r2k
a2 = 5. * a
c = np.tensordot(a, a2.conj(), [[1, 2, 3], [1, 2, 3]])
r2k(.5, a, a2, -1., c)
tri2full(c)
assert not c.any()



'Test ase.dft.wannier module with k-points.'
from __future__ import print_function

from ase.build import bulk
from ase.dft.wannier import Wannier

from gpaw import GPAW
from gpaw.mpi import world, serial_comm

si = bulk('Si', 'diamond', a=5.43)

k = 2
if 1:
    si.calc = GPAW(kpts=(k, k, k), mode='lcao', txt='Si-ibz.txt')
    #si.calc = GPAW(kpts=(k, k, k), mode='lcao')
    e1 = si.get_potential_energy()
    si.calc.write('Si-ibz', mode='all')
    #si.calc.set(symmetry='off', txt='Si-bz.txt')
    #si.calc.set(symmetry='off')
    #e2 = si.get_potential_energy()
    #si.calc.write('Si-bz', mode='all')
    #print e1, e2

def wan(calc):
    centers = [([0.125, 0.125, 0.125], 0, 1.5),
               ([0.125, 0.625, 0.125], 0, 1.5),
               ([0.125, 0.125, 0.625], 0, 1.5),
               ([0.625, 0.125, 0.125], 0, 1.5)]
    w = Wannier(4, calc,
                nbands=4,
                verbose=False,
                initialwannier=centers)
    w.localize()
    x = w.get_functional_value()
    centers = (w.get_centers(1) * k) % 1
    c = (centers - 0.125) * 2
    #print w.get_radii()  # broken! XXX
    assert abs(c.round() - c).max() < 0.03
    c = sorted(c.round().astype(int).tolist())
    assert c == [[0, 0, 0], [0, 0, 1], [0, 1, 0], [1, 0, 0]]
    if 0:
        from ase.visualize import view
        from ase import Atoms
        watoms = calc.atoms + Atoms(symbols='X4',
                                    scaled_positions=centers,
                                    cell=calc.atoms.cell)
        view(watoms)
    return x

calc1 = GPAW('Si-bz.gpw', txt=None, communicator=serial_comm)
# do full lcao initialization
#print('Re-Init LCAO calc')
_s = calc1.get_atoms()
calc1.set_positions(_s)
calc1.wfs.eigensolver.iterate(calc1.hamiltonian, calc1.wfs)
x1 = wan(calc1)
    
#print x1
x2 = 5.76962685681      # from FD mode
assert abs(x1 - x2) < 0.001
#assert abs(x1 - 9.71) < 0.01

world.barrier()



from __future__ import print_function
import sys

import numpy as np
from ase.dft.dos import DOS, linear_tetrahedron_integration as lti
from ase.units import Ha

from gpaw import GPAW
from gpaw.utilities.dos import raw_orbital_LDOS, delta


class CLICommand:
    """Calculate (projected) density of states from gpw-file."""

    @staticmethod
    def add_arguments(parser):
        add = parser.add_argument
        add('gpw', metavar='gpw-file')
        add('csv', nargs='?', metavar='csv-file')
        add('-p', '--plot', action='store_true',
            help='Plot the DOS.')
        add('-i', '--integrated', action='store_true',
            help='Calculate integrated DOS.')
        add('-w', '--width', type=float, default=0.1,
            help='Width of Gaussian.  Use 0.0 to use linear tetrahedron '
            'interpolation.')
        add('-a', '--atom', help='Project onto atoms: "Cu-spd,H-s" or use '
            'atom indices "12-spdf".')
        add('-t', '--total', action='store_true',
            help='Show both PDOS and total DOS.')
        add('-r', '--range', nargs=2, metavar=('emin', 'emax'),
            help='Energy range in eV relative to Fermi level.')
        add('-n', '--points', type=int, default=400, help='Number of points.')

    @staticmethod
    def run(args):
        if args.range is None:
            emin = None
            emax = None
        else:
            emin, emax = (float(e) for e in args.range)
        dos(args.gpw, args.plot, args.csv, args.width, args.integrated,
            args.atom, emin, emax, args.points, args.total)


def dos(filename, plot=False, output='dos.csv', width=0.1, integrated=False,
        projection=None, emin=None, emax=None, npoints=400, show_total=None):
    """Calculate density of states.

    filename: str
        Name of restart-file.
    plot: bool
        Show a plot.
    output: str
        Name of CSV output file.
    width: float
        Width of Gaussians.
    integrated: bool
        Calculate integrated DOS.

    """
    calc = GPAW(filename, txt=None)
    dos = DOS(calc, width, (emin, emax), npoints)

    nspins = calc.get_number_of_spins()
    spinlabels = [''] if nspins == 1 else [' up', ' dn']

    if projection is None or show_total:
        D = [dos.get_dos(spin) for spin in range(nspins)]
        labels = ['DOS' + sl for sl in spinlabels]
    else:
        D = []
        labels = []
    if projection is not None:
        for p in projection.split(','):
            s, ll = p.split('-')
            if s.isdigit():
                A = [int(s)]
                s = '#' + s
            else:
                A = [a for a, symbol in
                     enumerate(calc.atoms.get_chemical_symbols())
                     if symbol == s]
                if not A:
                    raise ValueError('No such atom: ' + s)
            for spin in range(nspins):
                for l in ll:
                    d = 0.0
                    for a in A:
                        d += ldos(calc, a, spin, l, width, dos.energies)
                    labels.append(s + '-' + l + spinlabels[spin])
                    D.append(d)

    if integrated:
        de = dos.energies[1] - dos.energies[0]
        dos.energies += de / 2
        D = [np.cumsum(d) * de for d in D]
        ylabel = 'iDOS [electrons]'
    else:
        ylabel = 'DOS [electrons/eV]'

    if output:
        fd = sys.stdout if output == '-' else open(output, 'w')
        for x in zip(dos.energies, *D):
            print(*x, sep=', ', file=fd)
        if output != '-':
            fd.close()
    if plot:
        import matplotlib.pyplot as plt
        for y, label in zip(D, labels):
            plt.plot(dos.energies, y, label=label)
        plt.legend()
        plt.ylabel(ylabel)
        plt.xlabel(r'$\epsilon-\epsilon_F$ [eV]')
        plt.show()


def ldos(calc, a, spin, l, width, energies):
    eigs, weights = raw_orbital_LDOS(calc, a, spin, l)
    eigs *= Ha
    eigs -= calc.get_fermi_level()
    if width > 0.0:
        dos = 0.0
        for e, w in zip(eigs, weights):
            dos += w * delta(energies, e, width)
    else:
        kd = calc.wfs.kd
        eigs.shape = (kd.nibzkpts, -1)
        eigs = eigs[kd.bz2ibz_k]
        eigs.shape = tuple(kd.N_c) + (-1,)
        weights.shape = (kd.nibzkpts, -1)
        weights /= kd.weight_k[:, np.newaxis]
        w = weights[kd.bz2ibz_k]
        w.shape = tuple(kd.N_c) + (-1,)
        dos = lti(calc.atoms.cell, eigs, energies, w)
    return dos



import numpy as np
from ase import Atoms
from gpaw import GPAW
from gpaw.tddft import TDDFT
from gpaw.inducedfield.inducedfield_base import BaseInducedField
from gpaw.inducedfield.inducedfield_tddft import TDDFTInducedField
from gpaw.poisson import PoissonSolver
from gpaw.test import equal

do_print_values = False  # Use this for printing the reference values
poisson_eps = 1e-12
density_eps = 1e-6

# PoissonSolver
poissonsolver = PoissonSolver('fd', eps=poisson_eps)

# Na2 cluster
atoms = Atoms(symbols='Na2',
              positions=[(0, 0, 0), (3.0, 0, 0)],
              pbc=False)
atoms.center(vacuum=3.0)

# Standard ground state calculation
calc = GPAW(nbands=2, h=0.6, setups={'Na': '1'}, poissonsolver=poissonsolver,
            convergence={'density': density_eps})
atoms.set_calculator(calc)
energy = atoms.get_potential_energy()
calc.write('na2_gs.gpw', mode='all')

# Standard time-propagation initialization
time_step = 10.0
iterations = 20
kick_strength = [1.0e-3, 1.0e-3, 0.0]
td_calc = TDDFT('na2_gs.gpw')
td_calc.absorption_kick(kick_strength=kick_strength)

# Create and attach InducedField object
frequencies = [1.0, 2.08]     # Frequencies of interest in eV
folding = 'Gauss'             # Folding function
width = 0.1                   # Line width for folding in eV
ind = TDDFTInducedField(paw=td_calc,
                        frequencies=frequencies,
                        folding=folding,
                        width=width,
                        restart_file='na2_td.ind')

# Propagate as usual
td_calc.propagate(time_step, iterations // 2, 'na2_td_dm.dat', 'na2_td.gpw')

# Save TDDFT and InducedField objects
td_calc.write('na2_td.gpw', mode='all')
ind.write('na2_td.ind')
ind.paw = None

# Restart and continue
td_calc = TDDFT('na2_td.gpw')

# Load and attach InducedField object
ind = TDDFTInducedField(filename='na2_td.ind',
                        paw=td_calc,
                        restart_file='na2_td.ind')

# Continue propagation as usual
td_calc.propagate(time_step, iterations // 2, 'na2_td_dm.dat', 'na2_td.gpw')

# Calculate induced electric field
ind.calculate_induced_field(gridrefinement=2,
                            from_density='comp',
                            poissonsolver=poissonsolver,
                            extend_N_cd=3 * np.ones((3, 2), int),
                            deextend=True)

# Save
ind.write('na2_td_field.ind', 'all')
ind.paw = None
td_calc = None
ind = None

# Read data (test also field data I/O)
ind = BaseInducedField(filename='na2_td_field.ind',
                       readmode='field')

# Estimate tolerance (worst case error accumulation)
tol = (iterations * ind.fieldgd.integrate(ind.fieldgd.zeros() + 1.0) *
       max(density_eps, np.sqrt(poisson_eps)))
# tol = 0.038905993684
if do_print_values:
    print('tol = %.12f' % tol)

# Test
val1 = ind.fieldgd.integrate(ind.Ffe_wg[0])
val2 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[0][0]))
val3 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[0][1]))
val4 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[0][2]))
val5 = ind.fieldgd.integrate(ind.Ffe_wg[1])
val6 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[1][0]))
val7 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[1][1]))
val8 = ind.fieldgd.integrate(np.abs(ind.Fef_wvg[1][2]))

if do_print_values:
    i = 1

    def equal(x, y, tol):  # noqa
        global i
        print("equal(val%d, %20.12f, tol)" % (i, x))
        i += 1

equal(val1, 1926.232999117403, tol)
equal(val2, 0.427606450419, tol)
equal(val3, 0.565823985683, tol)
equal(val4, 0.372493489423, tol)
equal(val5, 1945.618902611449, tol)
equal(val6, 0.423899965987, tol)
equal(val7, 0.560882533828, tol)
equal(val8, 0.369203021329, tol)



from __future__ import print_function

from ase import Atoms
from gpaw import GPAW
from gpaw.test import equal

from gpaw.external import ConstantPotential

cp = ConstantPotential()

sc = 2.9
R = 0.7  # approx. experimental bond length
R = 1.0
a = 2 * sc
c = 3 * sc
at = 'H'
H2 = Atoms('HH', [(a / 2, a / 2, (c - R) / 2),
                  (a / 2, a / 2, (c + R) / 2)],
           cell=(a, a, c),
           pbc=False)

txt = None

convergence = {'eigenstates': 1.e-4 * 40 * 1.5**3,
               'density': 1.e-2,
               'energy': 0.1}

# without potential
if True:
    if txt:
        print('\n################## no potential')
    c00 = GPAW(h=0.3, nbands=-1,
               convergence=convergence,
               txt=txt)
    c00.calculate(H2)
    eps00_n = c00.get_eigenvalues()

# 0 potential
if True:
    if txt:
        print('\n################## 0 potential')
    cp0 = ConstantPotential(0.0)
    c01 = GPAW(h=0.3, nbands=-2, external=cp0,
               convergence=convergence,
               txt=txt)
    c01.calculate(H2)

# 1 potential
if True:
    if txt:
        print('################## 1 potential')
    cp1 = ConstantPotential(-1.0)
    c1 = GPAW(h=0.3, nbands=-2, external=cp1,
              convergence=convergence,
              txt=txt)
    c1.calculate(H2)

for i in range(c00.get_number_of_bands()):
    f00 = c00.get_occupation_numbers()[i]
    if f00 > 0.01:
        e00 = c00.get_eigenvalues()[i]
        e1 = c1.get_eigenvalues()[i]
        print('Eigenvalues no pot, expected, error=',
              e00, e1 + 1, e00 - e1 - 1)
        equal(e00, e1 + 1., 0.008)

E_c00 = c00.get_potential_energy()
E_c1 = c1.get_potential_energy()
DeltaE = E_c00 - E_c1
equal(DeltaE, 0, 0.002)



from gpaw.solvation.cavity import get_pbc_positions
from ase.build import molecule
from ase.units import Bohr
import numpy as np


def get_nrepeats(nrepeats_c):
    return np.prod(np.array(nrepeats_c) * 2 + 1)


def check_valid_repeat(pos_v, rpos_v, nrepeats_c, cell_cv):
    nrepeats_c = np.array(nrepeats_c)
    diff_v = rpos_v - pos_v
    actual_nrepeats_c = np.dot(diff_v, np.linalg.inv(cell_cv))
    r_actual_nrepeats_c = np.around(actual_nrepeats_c)
    # check for integer multiple
    assert np.allclose(r_actual_nrepeats_c, actual_nrepeats_c)
    actual_nrepeats_c = np.abs(r_actual_nrepeats_c.astype(int))
    # check for too large repeats
    assert (actual_nrepeats_c <= nrepeats_c).all()


def check(pos_av, rpos_aav, nrepeats_c, cell_cv):
    assert len(pos_av) == len(rpos_aav)
    total_repeats = get_nrepeats(nrepeats_c)
    for a, pos_v in enumerate(pos_av):
        rpos_av = rpos_aav[a]
        assert len(rpos_av) == total_repeats
        for rpos_v in rpos_av:
            check_valid_repeat(pos_v, rpos_v, nrepeats_c, cell_cv)


cells = (
    ((10., .0, .0), (.0, 12., .0), (.0, .0, 12.)),  # orthogonal
    ((10., 1., .0), (.0, 12., 1.), (1., .0, 11.5)),  # non-orthogonal
    )

for cell in cells:
    atoms = molecule('H2O')
    atoms.center(vacuum=5.)
    atoms.set_cell(cell)
    pos_av = atoms.positions / Bohr
    cell_cv = atoms.get_cell() / Bohr
    Rcell_c = np.sqrt(np.sum(cell_cv ** 2, axis=1))

    # non periodic should never repeat
    atoms.pbc = np.zeros((3, ), dtype=bool)
    pos_aav = get_pbc_positions(atoms, 1e10 / Bohr)
    check(pos_av, pos_aav, (0, 0, 0), cell_cv)

    # periodic, zero cutoff should not repeat
    atoms.pbc = np.ones((3, ), dtype=bool)
    pos_aav = get_pbc_positions(atoms, .0 / Bohr)
    check(pos_av, pos_aav, (0, 0, 0), cell_cv)

    # periodic, cutoff <= cell size should repeat once
    atoms.pbc = np.ones((3, ), dtype=bool)
    pos_aav = get_pbc_positions(atoms, 0.99 * Rcell_c.min())
    check(pos_av, pos_aav, (1, 1, 1), cell_cv)

    # periodic, cutoff > cell size should repeat twice
    atoms.pbc = np.ones((3, ), dtype=bool)
    pos_aav = get_pbc_positions(atoms, 1.01 * Rcell_c.max())
    check(pos_av, pos_aav, (2, 2, 2), cell_cv)

    # mixed bc, cutoff > 2 * cell size should repeat three times
    for i in range(8):
        pbc = np.array([int(p) for p in np.binary_repr(i, 3)])
        atoms.pbc = pbc
        pos_aav = get_pbc_positions(atoms, 2.01 * Rcell_c.max())
        check(pos_av, pos_aav, pbc * 3, cell_cv)
